/* 
   ASCII.I
   Routines for reading numerical data from files in ascii format

   $Id: asciilib.i,v 1.14 2004/02/24 20:15:15 drory Exp $
*/

/*----------------------------------------------------------------------
 * @(#) ascii.i: Routines for reading ascii numerical data from files
 *
 * Niv Drory, drory@usm.uni-muenchen.de
 * 12.01.99   first release.
 * 16.04.99   new read_numbers routine, performance improved. 
 * 04.10.99   read_columns_mixed added. support for reading non-numerical
 *            columnar data
 * 24.10.99   improved detection of non-numerical data for read_columns_mixed
 *            should distinguish correctly in any case now.
 * 27.04.00   line ranges added. 
 *
 */

require, "string.i"


func read_columns( file, 
                      c1, &d1, c2, &d2, c3, &d3, c4, &d4, c5, &d5, c6, &d6,
                      c7, &d7, c8, &d8, c9, &d9, c10, &d10, c11, &d11, lines= )
/* DOCUMENT
   Read columns of data out of an ascii file.
   At most 11 columns can be read at once! Data is written to newly allocated
   arrays. 
   Column separators are  whitespace. Lines beginning with anything but a 
   number are treated as comments, empty lines are ignored.
   Only numerical data is alowed in the file, except for comment lines.
   The keyword lines= when specified gives the range of lines to consider
   (vector of legnth 2).

   SEE ALSO read_columns_mixed for a slower version which allows columns
   of non numerical data.
*/
{

   /* open the file */
   if( !(f=open( file, "r" )) )
      error, "[read_columns] : can't open file \'"+file+"\'!";

   numbers = read_numbers(f, lines=lines);
  
   close, f;

   if( numberof(numbers) == 0 ) {
      d1 = []; if(is_void(c2)) return;
      d2 = []; if(is_void(c3)) return; 
      d3 = []; if(is_void(c4)) return; 
      d4 = []; if(is_void(c5)) return; 
      d5 = []; if(is_void(c6)) return; 
      d6 = []; if(is_void(c7)) return; 
      d7 = []; if(is_void(c8)) return; 
      d8 = []; if(is_void(c9)) return; 
      d9 = []; if(is_void(c10)) return;
      d10 = []; if(is_void(c11)) return;
      d11 = [];   
      return;
   }

   d1 = numbers(c1,); if(is_void(c2)) return;
   d2 = numbers(c2,); if(is_void(c3)) return; 
   d3 = numbers(c3,); if(is_void(c4)) return; 
   d4 = numbers(c4,); if(is_void(c5)) return; 
   d5 = numbers(c5,); if(is_void(c6)) return; 
   d6 = numbers(c6,); if(is_void(c7)) return; 
   d7 = numbers(c7,); if(is_void(c8)) return; 
   d8 = numbers(c8,); if(is_void(c9)) return; 
   d9 = numbers(c9,); if(is_void(c10)) return;
   d10 = numbers(c10,); if(is_void(c11)) return;
   d11 = numbers(c11,);
}



func read_columns_mixed( file, 
			 c1, &d1, c2, &d2, c3, &d3, c4, &d4, c5, &d5, c6, &d6,
			 c7, &d7, c8, &d8, c9, &d9, c10, &d10, c11, &d11, 
			 lines= )
/* DOCUMENT
   Read columns of data out of an ascii file.
   At most 11 columns can be read at once! Data is written to newly allocated
   arrays. 
   Column separators are  whitespace. Lines beginning with '#' are treated 
   as comment lines, empty lines are ignored.
   The file may contain any numercal or string data as columns.
   The keyword lines= when specified gives the range of lines to consider
   (vector of legnth 2).

   SEE ALSO read_columns for a much faster version which works for files
   containing only numerical data.
*/
{
   /* open the file */
   if( !(f=open( file, "r" )) )
      error, "[read_columns] : can't open file \'"+file+"\'!";
  
   d = read_ascii( f, lines=lines );
   close, f;

   if( numberof(d) == 0 ) {
      d1 = []; if(is_void(c2)) return;
      d2 = []; if(is_void(c3)) return; 
      d3 = []; if(is_void(c4)) return; 
      d4 = []; if(is_void(c5)) return; 
      d5 = []; if(is_void(c6)) return; 
      d6 = []; if(is_void(c7)) return; 
      d7 = []; if(is_void(c8)) return; 
      d8 = []; if(is_void(c9)) return; 
      d9 = []; if(is_void(c10)) return;
      d10 = []; if(is_void(c11)) return;
      d11 = [];   
      return;
   }
   d1 = *d(c1);   if(is_void(c2))  return;
   d2 = *d(c2);   if(is_void(c3))  return;
   d3 = *d(c3);   if(is_void(c4))  return;
   d4 = *d(c4);   if(is_void(c5))  return;
   d5 = *d(c5);   if(is_void(c6))  return;
   d6 = *d(c6);   if(is_void(c7))  return;
   d7 = *d(c7);   if(is_void(c8))  return;
   d8 = *d(c8);   if(is_void(c9))  return;
   d9 = *d(c9);   if(is_void(c10)) return;
   d10 = *d(c10); if(is_void(c11)) return;
   d11 = *d(c11);
}



func read_ascii( file, lines= )
/* DOCUMENT
   data = read_ascii( file );

   Read ascii data out of a file. Returns array of pointers to columns
   of read data. Columns may be numerical, in which case they are 
   returned as a float array, or strings
   The keyword lines= when specified gives the range of lines to consider
   (vector of legnth 2).

*/
{
   /* first, read lines and collect them in a list (fast) */
   if( is_void(lines) )
      lines=[0,100000000];
   nlines = lines(2)-lines(1);

   n = 0;
   l = _lst();
   for( i=1; i<lines(1); i++ )
      lall = rdline(f);

   while( (line = rdline( f )) ) {
      line = strtrim(line);
   
      if( !strlen(line) || strpart( line, 1:1 ) == "#" ) 
         continue; /* empty or comment */

      ++n;
      l = _cat( line, l );
      if( n > nlines )
         break;
   }
   if(!_car(l)) return [];

   /* put tokenized lines in a ncolumns x nlines array of strings */
   lastline = tokenize( _car(l) );
   ncols = numberof( lastline );
   filelines = array( string, ncols, n );
   for( i=n ; _car(l) ; l=_cdr(l) )
      filelines(,i--) = tokenize( _car(l) );

   pointers  = array( pointer, ncols );
   //firstchar = strpart(lastline, 1:1);
   //lastchar  = strpart(lastline, 0:0);

   /* read colums out of matrix and convert to appropriate type */
   for( i=1; i<=ncols; i++ ) {
      //if( strmatch( "1234567890+-.", firstchar(i) ) &
      //    strmatch( "1234567890.", lastchar(i) ) ) {
      if( is_number( lastline(i)) ) {
         c = array( double, n );
         sread, filelines(i,), format="%f", c;
         pointers(i) = &c;
      } else {
         c = array( string, n );
         sread, filelines(i,), format="%s", c;
         pointers(i) = &c;
      }
   }
   return pointers;
}


func tokenize( line, delimiters )
/* DOCUMENT
   tokens = tokenize( line, delimiters )

   break up line into an array of tokens
*/
{
   if( is_void( delimiters ) )
      delimiters = " \t\n";
   a = array( string, 256 );
   i=0;
   for( ;; ) {
      tok = strtok( line, delimiters );
      line = tok(2,);
      a(++i) = tok(1,);
      if( !line )
         return a(1:i);
   }
}

func is_number( str )
{
   /* convert to array of char and get rid of null char at end */
   s = *pointer( str );
   n = numberof( s );
   s = s(1:n-1);
   n--;

   i = 1;
   if( s(i) == '.' ) /* only digit may follow a . */
   {
      i++;
      if( i>n ) return 0;
      if( !is_digit(s(i)) )
         return 0;
   }
  
   if( s(i)=='+' || s(i)=='-' || is_digit(s(i)) ) {
      i++;
      /* digits */
      while( i<=n && is_digit(s(i)) )
         i++;
      if( i>n )
         return 1;
    
      /* first non digit */
      if( s(i)=='.' )
         i++; 
      if( i>n )
         return 1;
    
      // eat digits again
      while( i<=n && is_digit(s(i)) )
         i++;
      if( i>n )
         return 1;

      if( s(i)=='e' || s(i)=='E' ) {
         i++; /* e or E */

         if( i>n )
            return 0;
      
         if( s(i)=='+' || s(i)=='-' )
            i++; 
         if( i>n )
            return 0;
      } else {
         if( !is_digit(s(i)) )
            return 0;
      }
    
      /* digits */
      while( i<=n && is_digit(s(i)) )
         i++;
      if( i>n )
         return 1;
   }
   return 0;
}

func is_digit(s)
{
   return (s>='0' & s<='9');
}

func read_numbers( file, lines= )
/* DOCUMENT data= read_numbers(name, lines=)
   data= read_numbers(file, lines=)
   read ascii numeric data in columns from file NAME, or the already
   open file FILE.
   The result is a NCOLUMNS-by-NLINES array of doubles.

   Data are read as double values arranged in columns separated
   by any number of spaces or tabs.  Comments starting with a "#"
   or any other character which is not part of a number are ignored
   up to the end-of-line.  Blank lines are ignored.
   The first non-blank/commented line gives the number of values per
   column, for subsequent lines.  Subsequent lines must have the
   same number of columns -- blanks in columns are not permitted,
   use 0.0 instead.  However, minimal error checking is performed,
   and if the data is not really in columns, asciiRead can silently
   fail to interpret your file as you would scanning it by eye.

   The read operation will be much faster if the number of commented
   lines is relatively small.  Blank lines cost nothing, while a line
   containing just a "#" is expensive.

   The keyword lines= when specified gives the range of lines to consider
   (vector of legnth 2).

   SEE ALSO: read
*/
{
   /* open the file if it's not already open */
   if (structof(file)==string) 
   {
      file= open(file);
      closefile=1;
   }
   

   if( is_void(lines) )
      lines=[0,100000000];
   nlines = lines(2)-lines(1)+1;

   for( i=1; i<lines(1); i++ )
      lall = rdline(file);

   /* read lines one at a time until the "model" line which
    * determines the number of columns is discovered
    * assume the number of columns is less than 550 */
   x= array(0.0, 550);
   ncols= 0;
   while ((line= rdline(file))) {
      if( strlen(line) && !strmatch( line, "#" ) ) {
         ncols= sread(line, x);
         if (ncols) break;          /* got a line with numbers */
      }
   }
   if (!ncols) 
   {
      if( closefile )
         close, file;     
      return [];
   }
  

   nrows= 1;
   list= _lst([x(1:ncols)]);
   x= array(0.0, ncols, 10000/ncols + 1);
   for(;;) {
      /* try to grab at least 10000 numbers from the file
       * blank lines will be skipped, but any comments will
       * interrupt the read */
      n= read(file, x);
      if (!n) {
         /* if didn't get any, drop back to reading comments one
          * line at a time until we get some more numbers */
         while ((line= rdline(file))) {
            n= sread(line, x);
            if (n) break;
         }
         if (!line) break;    /* rdline detected end-of-file, n==0 too */
      }
      if (n%ncols) error, "data is not in columns";
      n/= ncols;

      /* grow the list the fast way, adding new values to its head
       * (adding to the tail would make growth an n^2 proposition,
       *  as would using the grow function) */
      list= _cat(x(,1:n), list);
      nrows+= n;
   }

   /* pop chunks off list and reassemble result */
   x= array(0.0, ncols, nrows );

   for (i=nrows ; list ; list=_cdr(list)) {
      n= numberof(_car(list))/ncols;
      x(,i-n+1:i) = _car(list);
      i-= n;
   }

   last=nlines < nrows ? nlines : nrows;
   if( closefile )
      close, file;
  
   return x(,1:last);
}



__raw_line__ = [];  // line buffer for read_numbers_raw()

func reset_read_numbers_raw(x)
/* DOCUMENT 
   reset_read_numbers_raw;
   
   Reset the global line buffer for read_numbers_raw().
   Should be called before calling read_numbers_raw() for the first time
   on a file.
   
   SEE ALSO
   read_numbers_raw()
*/
{
   extern __raw_line__;
   __raw_line__ = [];
}


func read_numbers_raw(f, &n0, &n1, &n2, &n3, &n4, &n5, &n6, &n7, &n8, &n9)
/* DOCUMENT 
   read_numbers_raw, f, n0, n1, n2, ...
   
   grabs the next numbers n0, n1, n2, ... from file f, skipping over
   any whitespace, comma, semicolon, or colon delimited tokens which
   are not numbers. At most ten Ns are allowed. Any of the Ns can be 
   either a scalar or an array.
   
   *** WARNING ***  THIS FUNCTION IS NOT REENTRANT. Only one file
   can be read at a time. 
   
   SEE ALSO
   reset_read_numbers_raw()
*/
{
   _rnr_worker, f, n0;   if (is_void(n1)) return;
   _rnr_worker, f, n1;   if (is_void(n2)) return;
   _rnr_worker, f, n2;   if (is_void(n3)) return;
   _rnr_worker, f, n3;   if (is_void(n4)) return;
   _rnr_worker, f, n4;   if (is_void(n5)) return;
   _rnr_worker, f, n5;   if (is_void(n6)) return;
   _rnr_worker, f, n6;   if (is_void(n7)) return;
   _rnr_worker, f, n7;   if (is_void(n8)) return;
   _rnr_worker, f, n8;   if (is_void(n9)) return;
   _rnr_worker, f, n9;
}


func _rnr_worker( file, &var )
{
   extern __raw_line__;

   n = numberof( var );
   if( indirect = dimsof(var)(1) )
      value = structof(var)();

   i = 1;
   for(;;)
   {
      while( __raw_line__ )
      {
         tok = strtok( __raw_line__, ",;: \t" );
         __raw_line__ = tok(2);
         if( strlen( tok(1) ) )
         {
            if( indirect )
            {
               sread, tok(1), value;
               var(i) = value;
            }
            else
            {
               sread, tok(1), var;
            }
            i++;
            if( i>n )
               return;
         }
      }
      __raw_line__ = rdline( file );
      while( __raw_line__(1) == '#' )
         __raw_line__ = rdline( file );
      
      if( !__raw_line__ )
         error, "Premature end of file in read_numbers_raw()";
   }
}

