include, "ncops.i";

func c3a_tlat( file, &y, &t )
/* DOCUMENT:

   c3a_tlat( file, &y, &t, test= )

   Computes the average zonal surface air temperature t (units: deg C)
   as a function of latitude y (units: degrees) from a CLIMBER-3alpha
   atmosphere history file using the last 30 years in the history
   output.

   Tested by comparing the output to the values computed by ferret on
   the same file.
 */

{
  // read atmosphere history file: 
  d = nc_openf( file, data, meta );

  // coordinates:
  yt  = data.yt_j;

  // return y coordinate array:
  y = yt;

  
  // loop through last 30 years of simulation:
  nc = meta.numrecs;
  ct = 0.;
  tl = y*0.;
  for( n=nc-29; n<=nc; n++ ){
    ct += 1.;
    jr, data, n;
    t1 = data.ts_ann;
    tl += t1(avg,);
  }
  tl /= ct;
 
  t = tl;

}

func c3a_aplat( file, &y, &ap )
/* DOCUMENT:

   c3a_aplat( file, &y, &ap, test= )

   Computes the average zonal planetary albedo (units: 1) as a
   function of latitude y (units: degrees) from a CLIMBER-3alpha
   atmosphere snapshot file. We do this on a snapshot rather than a
   history file because the history files set the planetary albedo in
   polar areas to NaN when averaging over the seasons (which is kind
   of stupid).

   Tested by comparing the output to the values computed by ferret on
   the same file.
 */

{
  // read atmosphere snapshot file: 
  d = nc_openf( file, data, meta );

  // coordinates:
  yt  = data.yt_j;

  // return y coordinate array:
  y = yt;

  // loop through the 12 months of the simulation:
  apl = ct = array( 0., 16, 24 );
  for( n=1; n<=12; n++ ){
    jr, data, n;
    ap1 = data.ap;
    good = (ap1>=0.);
    ct += good;
    apl += (good*ap1);
  }
  apl = apl(sum,)/ct(sum,);
 
  ap = apl;

}

func c3a_siv( file, &y, &v )
/* DOCUMENT:

   c3a_siv( file, &y, &v, test= )

   Computes the zonal-average sea-ice meridional velocity (units:
   cm/s) as a function of latitude y (units: degrees) from a
   CLIMBER-3alpha sea-ice snapshot file.

   Tested by comparing the output to the values computed by ferret on
   the same file.
 */

{
  // read sea-ice snapshot file: 
  d = nc_openf( file, data, meta );

  // coordinates:
  yt  = data.yt_j;
  yu  = data.yu_j;

  // return y coordinate array:
  y = yu(1:-1);

  // loop through the 12 months of the simulation:
  vl = ct = array( 0., 96, 47 );
  for( n=1; n<=12; n++ ){
    jr, data, n;
    v1 = data.vice(,1:-1,1);
    good = (v1>=-1.e4);
    ct += good;
    vl += (good*v1);
  }
  vl = vl(sum,)/ct(sum,);

  v = vl;

}


func c3a_hto( file, &y, &h, test= )
/* DOCUMENT:

   c3a_hto( file, &y, &h, test= )

   Computes the meridional ocean heat transport h (units: W) as a
   function of latitude y (units: degrees) from a CLIMBER-3alpha MOM3
   snapshot based on the heat flux at the interface to the
   atmosphere. Option test= sets a test mode in which an idealised
   symmetric profile for the heat fluxes is generated for tests of
   pixel offsets.

   Tested using pre-industrial climate simulations compared against
   modern estimates.
 */
{
 
  // read ocean snapshot file: 
  d = nc_openf( file, data, meta );

  // coordinates:
  yt  = data.yt_j;
  yu  = data.yu_j;

  // return yu coordinate array:
  y = yu(1:-1);

  // total surface are of Earth [m2], weights:
  a = 5.1e14;
  w = get_geo_weights(96,48);
  
  nc = meta.numrecs;
  h = yt*0.;
  for( n=1; n<=nc; n++ ){
    
    jr, data, n;

    // get ocean surface heat flux from data [cal/cm2/s]:
    hflx = data.hflx;
    // convert it to SI units [J/m2/s]:
    hflx *= 4.1868e4;

    // test option with idealised symmetric fluxes:
    if( !is_void(test) ){
      yt2 = yt(-:1:96,);
      hflx = 200.*cos(yt2/45.*pi)/w;
    }

    // construct ocean mask from the data array:
    mask = (hflx>-1.e30);

    // OK, now let's compute the surface heat flux [W] and sum over
    // all longitudes:
    heat = a*(w*mask*hflx)(sum,);
    
    // and now add up the values South of each latitude:
    for( m=1; m<=numberof(yt); m++ ){
      h(m) += sum(heat(1:m));
    }

  }
  
  h /= nc;
  h = h(1:-1);

}


func c3a_get_ocean_depth( void )
/* DOCUMENT:

   Usage: d = c3a_get_ocean_depth()

   Returns an array of CLIMBER-3alpha ocean depth levels (25 values,
   edges of the grid cells, unit meters).

 */
{
  return [0, 25, 50, 75, 100, 126.5082, 159.0119, 203.3592, 265.1063,
          349.3812, 460.7569, 603.1393, 779.6709, 992.6531, 1243.49,
          1532.653, 1859.671, 2223.14, 2620.757, 3049.381, 3505.106,
          3983.359, 4479.012, 4986.508, 5498.49];
}


func c3a_ocean_characteristics( depth_levels )
/* DOCUMENT:

   Usage: [depth,area,volume] = c3a_ocean_characteristics( depth_levels )

   Computes an area containing the the average ocean depth (in m), the
   global ocean area (in m2) and the total ocean volume (in m3) for an
   array of CLIMBER-3alpha ocean depth levels (dimensions [2, 96, 48])

 */
{

  // get absolute depth (in meters) of edges of grid cells:
  z = c3a_get_ocean_depth();

  zzz = depth_levels*0.;
  // convert level ids to depth in meters:
  for( n=1; n<=24; n++ ){
    w = where( depth_levels==n );
    if( numberof(w) )
      zzz(w) = z(n+1);
  }
 
  gsurf  = 5.10072e14;  // area of Earth's surface (in m2)
  weight = get_geo_weights( 96, 48 );
  ocean  = (depth_levels>0.); 
  gdepth = sum(ocean*weight*zzz)/sum(ocean*weight);
  garea  = sum(ocean*weight)/sum(weight)*gsurf;
  gvol   = sum(ocean*weight*zzz*gsurf)/(sum(ocean*weight));
  return [gdepth,garea,gvol];
     
}




func c3a_write_kmt( filename, data )
/* DOCUMENT:

   Usage: write_kmt, FILENAME, DATA

   Writes CLIMBER-3alpha ocean depth levels (stored in DATA with
   dimensions [2, 96, 48]) into a file FILENAME in the format used in
   kmt.dta.

 */
{
  data = int(data);

  f = open( filename, "w" );

  // first data block:
  write, f, "   98    50    24partial cells       \n\n\n";
  write, f, "      1     3     5     7     9    11    13    15    17    19    21    23    25    27    29    31    33    35    37    39\n";
  write, f, " 50   0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0";
  for( n=49; n>1; n-- ){
    write, f, format="  %2d ", n;
    for( m=1; m<=40; m++ )
      if( m<40 )
        write, f, format="%3d", data(m-1,n-1), linesize=125;
      else
        write, f, format="%3d\n", data(m-1,n-1), linesize=125;
  }
  write, f, "  1   0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0";

  // second data block:
  write, f, "\n\n";
  write, f, "     41    43    45    47    49    51    53    55    57    59    61    63    65    67    69    71    73    75    77    79\n";
  write, f, " 50   0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0";
  for( n=49; n>1; n-- ){
    write, f, format="  %2d ", n;
    for( m=41; m<=80; m++ )
      if( m<80 )
        write, f, format="%3d", data(m-1,n-1), linesize=125;
      else
        write, f, format="%3d\n", data(m-1,n-1), linesize=125;
  }
  write, f, "  1   0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0";

  // third data block:
  write, f, "\n\n";
  write, f, "     81    83    85    87    89    91    93    95    97";
  write, f, " 50   0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0";
  for( n=49; n>1; n-- ){
    write, f, format="  %2d ", n;
    for( m=81; m<=98; m++ )
      if( m<98 )
        write, f, format="%3d", data(m-1,n-1), linesize=125;
      else
        write, f, format="%3d\n", data(1,n-1), linesize=125;
  }
  write, f, "  1   0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0";

  close, f;

}

