require, "ncops.i";

/*
  Computes the transformation matrix from (lon,lat) [in degrees] to
  (x,y) for a Mollweide map projection.
 */
func compute_mollweide_transformation( lon, lat )
{

  // convert lon, lat to radians:
  lon *= pi/180.;
  lat *= pi/180.;

  // make lon a 2D array:
  lon2 = lon(,-:1:numberof(lat));

  // loop through all latitudes:
  theta = [];
  for( nlat=1; nlat<=numberof(lat); nlat++ ){
    // compute theta by Newton-Raphson iteration:
    grow, theta, comp_mollweide_theta( lat(nlat) );
  }

  // turn theta into a 2D matrix, expanding it to match the (lon,lat)
  // grid:
  theta2 = theta(-:1:numberof(lon),);

  // compute the x and y coordinate arrays:
  x2 = 2.*lon2*cos(theta2)/pi;
  y2 = sin(theta2);

  return [x2,y2];
    
}

// !!! _lat_ is expected in radians !!!
func comp_mollweide_theta( _lat_ )
{

  // special case: _lat_ = +/- pi/2  (+/- 90 degrees):
  halfpi = pi/2.;
  if( _lat_==halfpi )
    _theta = halfpi;
  else
    if( _lat_==-halfpi )
      _theta = -halfpi;
    else{
      // compute theta by Newton-Raphson iteration:
      init = 1;
      _theta = 0.;       // dummy!
      _theta_new = 42.;  // dummy!
      // compute theta to Yorick double precision:
      while( abs(_theta_new-_theta)>1.e-14 ){
        if( init ){
          _theta_new = _lat_;    // inital value for iteration
          init = 0;
          continue;
        }
        else{
          _theta = _theta_new;
          _theta_new = _theta -
            ( 2.*_theta +
              sin(2.*_theta) -
              pi*sin(_lat_) )/( 4.*cos(_theta)^2 );
        }
      }
    }

  return _theta;

}


// compute Mollweide transformation for arbitrary arrays of
// coordinates x and y (given in degrees)
func compute_mollweide_transformation_array( x, y )
{

  // convert x, y to radians:
  x *= pi/180.;
  y *= pi/180.;

  // loop through all points:
  theta = x*0.;
  for( nx=1; nx<=dimsof(x)(2); nx++ )
    for( ny=1; ny<=dimsof(x)(3); ny++ )
      // compute theta by Newton-Raphson iteration:
      theta(nx,ny) = comp_mollweide_theta( y(nx,ny) );
  
  // compute the transformed x and y coordinate arrays:
  x_m = 2.*x*cos(theta)/pi;
  y_m = sin(theta);

  return [x_m,y_m];
  
}


func plot_mollweide_grid( color=, width= )
{

  // compute Mollweide transformation matrix:
  m_grid = compute_mollweide_transformation( span(-180., 180., 361), span(-90., 90., 181 ) );

  // plot grid lines:
  for( n=1; n<=5; n++ ){
    plg, m_grid(,,2)(,1+n*30), m_grid(,,1)(,1+n*30),
      color=color, width=width;
    plg, m_grid(,,2)(1+n*60,), m_grid(,,1)(1+n*60,),
      color=color, width=width;
  }

}

func plot_mollweide_edge( color=, width= )
{

  // compute Mollweide transformation matrix:
  m_grid = compute_mollweide_transformation( span(-180., 180., 361), span(-90., 90., 181 ) );

  // plot edge lines:
  plg, m_grid(,,2)(0,), m_grid(,,1)(0,),
    color=color, width=width;
  plg, m_grid(,,2)(1,), m_grid(,,1)(1,),
    color=color, width=width;

}





func example_mollweide( void )
{
  
  // read file with CLIMBER-3alpha output:
  file = "~/Climate/Projects/ACE/LIP_Extinction/Data/pulse_rev_cs_1500ppm_5300GtC_125GtS_5ka_history_combined.nc";
  d = nc_openf( file, data, meta );

  // construct CLIMBER-3alpha atmosphere grid:
  lat_a = span( -90., +90., 25 );
  lon_a = span( -180., 180., 17 );

  // compute Mollweide transformation matrix:
  m_data = compute_mollweide_transformation( lon_a, lat_a );
  // manually adjust to avoid plotting artifacts near the poles:
  m_data(,1,2) = -1.01;
  m_data(,0,2) =  1.01;

  // plot a temperature map:
  window, style="mollweide.gs";
  fma;
  plf, data.ts_ann, m_data(,,2), m_data(,,1);

  plot_mollweide_grid, color=100*[1,1,1], width=1;

  plot_mollweide_edge, color="black", width=3;

  limits, -2.1, 2.1, -1.05, 1.05; 
  
  error;


}
