include, "YorickInclude/graphlib.i";          // graphics support
include, "YorickInclude/asciilib.i";          // ASCII I/O
include, "YorickInclude/geolib.i";            // grid weights etc.
include, "YorickInclude/climber3a.i";         // MOM3 depth levels etc.
include, "YorickInclude/mollweide.i";         // Mollweide map projections
include, "YorickInclude/lmfit2.i";            // function fitting
include, "YorickInclude/devonian_forcings.i"; // forcings and CO2 equivalents
include, "YorickInclude/archean.i";           // conversion solar luminosity-age
include, "YorickInclude/ssa.i";               // SSA smoothing


// --------------------------------------------------------------------
// MODEL PRE-PROCESSING: BUILDING INPUT TOPOGRAPHY FILES             //
// --------------------------------------------------------------------

func glac_proc_topo( flat= )
{

  // --------------------------------------------------------------------
  // CREATE ARRAYS
  // --------------------------------------------------------------------

  // construct CLIMBER-3alpha ocean grid:
  lat_o = span( -88.125, +88.125, 48 );
  lon_o = span( 1.875, 358.125, 96 );
  // 2D arrays:
  lat_o2 = lat_o(-:1:numberof(lon_o),);
  lon_o2 = lon_o(,-:1:numberof(lat_o));

  // construct CLIMBER-3alpha atmosphere grid:
  lat_a = span( -86.25, +86.25, 24 );
  lon_a = span( 11.25, 348.75, 16 );
  // 2D arrays:
  lat_a2 = lat_a(-:1:numberof(lon_a),);
  lon_a2 = lon_a(,-:1:numberof(lat_a));

  // only for test plot: atmosphere grid corners:
  lat_ac = span( -90., 90., 25 );
  lon_ac = span( 0., 360., 17 );
  // 2D arrays:
  lat_ac2 = lat_ac(-:1:numberof(lon_ac),);
  lon_ac2 = lon_ac(,-:1:numberof(lat_ac));



  // assign interpolated topography values to CLIMBER-3alpha ocean
  // depth levels:
     read_columns, "ModelInput/mom_depth_levels.dat", 
     1,k, 2,z, 4,z_top;
  if( is_void(flat) )
    elev_o = -3000. + random_n( dimsof(lat_o2) )*450.;
  else
    elev_o = array( -3000., dimsof(lat_o2) );
  depth = elev_o*0;
  for( n=1; n<=24; n++ ){
    w = where( elev_o<-z_top(n) );
    if( numberof(w) )
      depth(w) = n;
  }
  // set uniform depth at the South and North poles (as for the
  // Neoproterozoic), although it is probably not necessary:
  depth(,1)  = round(avg( depth(,1) ));
  depth(,48) = round(avg( depth(,48) ));

  // assign tree, grass and bare-land fractions on the CLIMBER-3alpha
  // atmosphere grid; all zero for aquaplanets:
  ft_a = fg_a = fb_a = fl_a = 0.*abs(lat_a2);


  // --------------------------------------------------------------------
  // WRITE OUTPUT FILES
  // --------------------------------------------------------------------

  // write ocean depth levels to file kmt.dta:
  if( is_void(flat) )
    out_kmt = "ModelInput/aquaplanet_kmt.dta";
  else
    out_kmt = "ModelInput/aquaplanet_flat_kmt.dta";
  c3a_write_kmt, out_kmt, depth;

  // write land orography data into ASCII file (longitudes are in
  // columns, units are converted from m to cm):
  oro_o = 0.*lat_o2;
  if( is_void(flat) )
    out_oro = "ModelInput/aquaplanet_oro.dat";
  else
    out_oro = "ModelInput/aquaplanet_flat_oro.dat";
  f = open( out_oro, "w" );
  write, f, format=" %6.4e", oro_o*oro_o, linesize=11*numberof(lon_o);
  close, f;
  
  // write tree, grass and bare-land fractions into ASCII file:
  if( is_void(flat) )
    out_veg = "ModelInput/aquaplanet_p2_vegetat_obs.dat";
  else
    out_veg = "ModelInput/aquaplanet_flat_p2_vegetat_obs.dat";
      f = open( out_veg, "w" );
  lsize = 9*numberof(lon_a);
  first=1;
  for( n=numberof(lat_a); n>=1; n-- ){
    write, f, format="%9.5f", abs(ft_a(,n)), linesize=lsize;
    write, f, format="%9.5f", abs(fg_a(,n)), linesize=lsize;
    for( m=1; m<=numberof(lon_a); m++ ){
      if( m<numberof(lon_a) )
        write, f, format="%9.5f", abs(fb_a(m,n)), linesize=lsize;
      else
        write, f, format="%9.5f\n", abs(fb_a(m,n)), linesize=lsize;
    }
  }
  close, f;


  // --------------------------------------------------------------------
  // MAKE TEST PLOTS
  // --------------------------------------------------------------------

  // MOM3 depth levels (ocean grid):
  window, 0, style="worldmap.gs";
  fma;
  colorbar_, -24, 0, viewport=[0.20,0.60,0.42,0.43], 
    nmajor=10, nminor=0, tlabel_b=1, horizontal=1,
      tlabel_height=12;
  pli, -depth,  -180, -90, 180, 90, cmin=-24, cmax=0.1;
  plm, lat_ac2, lon_ac2-180., color=[50,50,50], width=1;
  r = span( 0, 120, 239 ); grow, r, [255,120];
  g = span( 0, 120, 239 ); grow, g, [255,120];
  b = span( 50, 255, 239 ); grow, b, [255,120];
  palette, r,g,b;
  gridxy, base60=3;
  pltitle_height=14;
  pltitle, "Ocean model depth levels (kmt.dta, after adjustments)";
  xytitles, "Longitude", "Latitude", [0.01,0.02];
  pdf, "Plots/aquaplanet_t4_kmt";
  png, "Plots/aquaplanet_t4_kmt", dpi=600;


}

// --------------------------------------------------------------------
// MODEL POST-PROCESSING: GENERATE LOCAL MODEL OUTPUT FILES          //
// --------------------------------------------------------------------

// global diagnostics for the critical states for all time slices as
// used in Figure 2

func glac_diag_critical_states_global( checkco2= )
{

  // paths and last snapshot dates for critical state simulations:
  pbasedir = "/p/projects/ace/feulner/";
  //  psubdir  = "glac_critical_states/";
  psubdir = "";
  // read times and critical CO2 concentrations from file:
  read_columns, "ModelOutput/snowball_bifurcation_aquaplanet_final.dat",
    1,tcrit, 2,lcrit, 4,ccrit;
  tcrit = int(tcrit);

  // weight factors on atmosphere and ocean grids:
  wa = get_geo_weights( );
  wo = get_geo_weights( 96, 48 );

  // open output file:
  f = open( "ModelOutput/critical_states_characteristics.dat", "w" );
  write, f, "#  Created ", timestamp(), "by glac_diag_critical_states_global() in threshold_paper.i";
  write, f, "#";
  write, f, "#  Global diagnostics of critical states from Aquaplanet simulations.";
  write, f, "#";
  write, f, "#  Columns:";
  write, f, "#   1: Age (Ma)";
  write, f, "#   2: Solar constant (W/m2)";
  write, f, "#   3: CO2 concentration (ppmv)";
  write, f, "#   4: Number of model years";
  write, f, "#   5: Global sea-ice fraction";
  write, f, "#   6: Global and annual mean surface air temperature (°C)";
  write, f, "#   7: Planetary albedo";
  write, f, "#   8: Outgoing longwave radiation (OLR) (W/m2)";
  write, f, "#";

  
  // loop through all model experiments:
  temp = albp = olr = icel = [];
  // for( ne=7; ne<=7; ne++ ){
  for( ne=1; ne<=numberof(tcrit); ne++ ){

    // construct full path name using time and critical CO2 value:
    if( ccrit(ne)>1. )
      pcrit = pbasedir + psubdir + swrite( format="c3beta_aqua_%04dMa_%05dppm/",
                                           tcrit(ne), int(ccrit(ne)) );
    else
      pcrit = pbasedir + psubdir + swrite( format="c3beta_aqua_%04dMa_00000p%1dppm/",
                                           tcrit(ne), int(10.*ccrit(ne)) );


    // load ISIS and POTSDAM-2 history files:
    d = nc_openf( pcrit+"/history_isis.nc", data_si, meta_si );
    d = nc_openf( pcrit+"/history_p2.nc", data_p2, meta_p2 );

    // get number of records in history files:
    nrec1 = meta_p2.numrecs;

    // compute average 2D tmperature and sea-ice field for the last 30 years:
    ct1 = nco2 = 0.;
    temp1 = albp1 = olr1 = data_p2.ts_ann*0.;
    sice1 = data_si.con_g*0.;
    if( !is_void(checkco2) ){
      for( nr=nrec1; nr>=1; nr-- ){
        jr, data_p2, nr;
        nco2 += 1;
        if( nr==nrec1 )
          co20 = data_p2.co2;
        if( data_p2.co2!=co20 )
          break;
      }
    }
    for( nr=nrec1-99; nr<=nrec1; nr++ ){
      ct1 += 1.;
      jr, data_p2, nr;
      jr, data_si, nr;
      temp1 += data_p2.ts_ann;
      _albp1 = data_p2.ap_ann;
      _adef = ( _albp1 > 0. );
      albp1 += _albp1*_adef;
      olr1  += data_p2.fl3_ann;
      sice1 += data_si.con_g;
    }
    temp1 /= ct1;
    albp1 /= ct1;
    olr1  /= ct1;
    sice1 /= ct1;

    // compute global mean values:
    grow, temp, sum( temp1*wa )/sum( wa );
    grow, albp, sum( albp1*wa*_adef )/sum( wa*_adef );
    grow, olr,  sum( olr1*wa )/sum( wa );
    grow, fice, sum( sice1*wo )/sum( wo );


    
    // for test of script only:
    if( !is_void(checkco2) )
      write, swrite( format="%4d Ma, %5.0f ppm, SAT: %6.2f, ice fraction: %6.4f, years since last CO2 change: %4d",
                     tcrit(ne), ccrit(ne), temp(ne), fice(ne), int(nco2) );
    else
      write, swrite( format="%4d Ma, %5.0f ppm, SAT: %6.2f, ice fraction: %6.4f",
                     tcrit(ne), ccrit(ne), temp(ne), fice(ne) );

    // write arrays to ouptut file:  
    write, f, swrite( format="%4d   %6.1f   %7.1f   %5d   %8.6f   %8.4f   %6.4f   %6.2f", tcrit(ne), lcrit(ne), ccrit(ne), nrec1, fice(ne), temp(ne), albp(ne), olr(ne) );

    
  }

  // close output file:
  close, f;

}


// characteristics of all 900 Ma states at different CO2
// concentrations as used in Figure 4

func glac_diag_900Ma_states( void )
{

  // paths and last snapshot dates for critical state simulations:
  pbasedir = "/p/projects/ace/feulner/";
  psubdir = "";
  // list of CO2 concentrations:
  co2 = [230.,240.,250.,260.,270.,280.,290.,300.,350.,400.,500.,600.];
  time = int( array( 900, numberof(co2) ) );
  lum =  array( 1261., numberof(co2) )

  // weight factors on atmosphere and ocean grids:
  wa = get_geo_weights( );
  wo = get_geo_weights( 96, 48 );

  // open output file:
  f = open( "ModelOutput/0900Ma_states_characteristics.dat", "w" );
  write, f, "#  Created ", timestamp(), "by glac_diag_900Ma_states() in threshold_paper.i";
  write, f, "#";
  write, f, "#  Global diagnostics of 900 Ma states from Aquaplanet simulations.";
  write, f, "#";
  write, f, "#  Columns:";
  write, f, "#   1: Age (Ma)";
  write, f, "#   2: Solar constant (W/m2)";
  write, f, "#   3: CO2 concentration (ppmv)";
  write, f, "#   4: Number of model years";
  write, f, "#   5: Global sea-ice fraction";
  write, f, "#   6: Global and annual mean surface air temperature (°C)";
  write, f, "#";

  
  // loop through all model experiments:
  temp = icel = [];
  // for( ne=7; ne<=7; ne++ ){
  for( ne=1; ne<=numberof(co2); ne++ ){

    // construct full path name using CO2 values:
    path = pbasedir + psubdir + swrite( format="c3beta_aqua_0900Ma_%05dppm/",
                                        int(co2(ne)) );


    // load ISIS and POTSDAM-2 history files:
    d = nc_openf( path+"/history_isis.nc", data_si, meta_si );
    d = nc_openf( path+"/history_p2.nc", data_p2, meta_p2 );

    // get number of records in history files:
    nrec1 = meta_p2.numrecs;

    // compute average 2D tmperature and sea-ice field for the last 30 years:
    temp1 = sice1 = ct1 = nco2 = 0.;
    for( nr=nrec1-99; nr<=nrec1; nr++ ){
      ct1 += 1.;
      jr, data_p2, nr;
      jr, data_si, nr;
      temp1 += data_p2.ts_ann;
      sice1 += data_si.con_g;
    }
    temp1 /= ct1;
    sice1 /= ct1;

    // compute global mean temperature and sea-ice fraction:
    grow, temp, sum( temp1*wa )/sum( wa );
    grow, fice, sum( sice1*wo )/sum( wo );


    
    // for test of script only:
    write, swrite( format="%4d Ma, %5.0f ppm, SAT: %6.2f, ice fraction: %6.4f",
                   time(ne), co2(ne), temp(ne), fice(ne) );

    // write arrays to ouptut file:  
    write, f, swrite( format="%4d   %6.1f   %7.1f   %5d   %8.6f   %8.4f", time(ne),
                      lum(ne), co2(ne), nrec1, fice(ne), temp(ne) );

    
  }

  // close output file:
  close, f;

}



func glac_diag_hadley_states( void )
{

  // READ DATA:
  pbasedir = "/p/projects/ace/feulner/";
  // read times and critical state characteristics from files:
  local acrit, csnow, ccrit, ycrit, tcrit;
  read_columns, "ModelOutput/snowball_bifurcation_aquaplanet_final.dat",
    1,acrit, 2,lcrit, 3,csnow, 4,ccrit;
  acrit = int(acrit);
  read_columns, "ModelOutput/critical_states_characteristics.dat",
    4,ycrit, 6,tcrit;

  // identify Ferrel states:
  ferrel = ( tcrit>-10. );
  // for the Ferrel states, we have to use the run which falls
  // into a Snowball:
  co2 = ccrit;
  co2(where(ferrel)) = csnow(where(ferrel));

  // open output file:
  f = open( "ModelOutput/hadley_states_characteristics.dat", "w" );
  write, f, "#  Created ", timestamp(), "by glac_diag_hadley_states() in threshold_paper.i";
  write, f, "#";
  write, f, "#  Global diagnostics of (transient and critical) Hadley states from Aquaplanet simulations.";
  write, f, "#";
  write, f, "#  Columns:";
  write, f, "#   1: Age (Ma)";
  write, f, "#   2: Solar constant (W/m2)";
  write, f, "#   3: CO2 concentration (ppmv)";
  write, f, "#   4: Beginning of Hadley phase (model years)";
  write, f, "#   5: End of Hadley phase (model years)";
  write, f, "#   6: Type of state (0: transient, 1: equilibrium)";
  write, f, "#";

  
  // loop through all model experiments:
  fice = [];
  //  for( ne=1; ne<=2; ne++ ){
  for( ne=1; ne<=numberof(tcrit); ne++ ){

    // construct full path name using time and CO2 value:
    if( co2(ne)>1. )
      pcrit = pbasedir + swrite( format="c3beta_aqua_%04dMa_%05dppm/",
                                           acrit(ne), int(co2(ne)) );
    else
      pcrit = pbasedir + swrite( format="c3beta_aqua_%04dMa_00000p%1dppm/",
                                           acrit(ne), int(10.*co2(ne)) );


    // load sea-ice history file:
    d = nc_openf( pcrit+"/history_isis_mean.nc", data, meta );

    // get number of records in history files:
    nrec = meta.numrecs;

    // compute global and annual mean sea-ice fraction for the last
    // 1000 years:
    fice = [];
    for( nr=1; nr<=nrec; nr++ ){
      jr, data, nr;
      grow, fice,  data.con_g;
    }

    // identify first and last model year for the Hadley states:
    w = where( fice>0.499 & fice<0.501 );
    
    // write arrays to ouptut file:  
    write, f, swrite( format="%4d   %6.1f   %7.1f   %5d   %5d   %1d", acrit(ne), lcrit(ne), co2(ne), w(1), w(0), !ferrel(ne) );
    
  }

  // close output file:
  close, f;

}





// --------------------------------------------------------------------
// GENERATE THE FIGURES FOR THE PAPER                                //
// --------------------------------------------------------------------



// Vastly improved and modified copy of plot_glaciation_threshold_aqua
// from threshold.i (3.3.2021)

func glac_figure1( builds= )
{

  // define the plot limits on the x-axis (important for labels on top
  // of the plot!):
  xl = 0.735;
  xr = 1.015;

  // define color for proxy values:
  pcol = [125,125,125];

  // define half-width for arrow heads and error bars:
  dl = 0.0015;

  window, style="notsoflat.gs";
  fma;


  // mark important ages on top of the plot:
  s = array( 0, [2,1,1] );
  //  pli, bytscl( s, cmin=0, cmax=1),
  //    gough_age_to_L(1.0), 0.5, gough_age_to_L(0.542), 1.0;
  ages = [4.0,3.5,3.0,2.5,2.0,1.5,1.0,0.5,0.];
  //  nages = ["3.8","2.5","0.5","0.0"];
  nages = ["4.0","3.5","3.0","2.5","2.0","1.5","1.0","0.5","0.0"];
  // convert this into relative solar luminosities:
  lages = gough_age_to_L( ages );
  pldj, lages, array( 1./1.356, numberof(ages) ), lages, array( 1., numberof(ages) );
  for( n=1; n<=numberof(ages); n++ )
    // viewport= { 0.14, 0.75, 0.45, 0.90 }
    plt, nages(n), 0.14+(lages(n)-xl)/(xr-xl)*(0.75-0.14), 0.905, 
      tosys=0, height=18, justify="CB";
  plg, [1.,1.], [xl,xr];
  ages = [3.8,2.5,0.542,0.];
  lages = gough_age_to_L( ages );
  // stage colors:
  // Hadean:
  ccc = char([174,2,126]);
  plf, array(ccc,2,2), [[1.99,1.99],[1.03,1.03]],
    [[xl,lages(1)],[xl,lages(1)]];
  // Archean:
  ccc = char([240,4,127]);
  plf, array(ccc,2,2), [[1.99,1.99],[1.03,1.03]],
    [[lages(1),lages(2)],[lages(1),lages(2)]];
  // Proterozoic:
  ccc = char([247,53,99]);
  plf, array(ccc,2,2), [[1.99,1.99],[1.03,1.03]],
    [[lages(2),lages(3)],[lages(2),lages(3)]];
  // Phanerozoic:
  ccc = char([154,216,221]);
  plf, array(ccc,2,2), [[1.99,1.99],[1.03,1.03]],
    [[lages(3),lages(4)],[lages(3),lages(4)]];
  pldj, lages, array( 1.99, 4 ), lages, array( 1., 4 );
  plt, "Archean", (lages(1)+lages(2))/2., 1.445, 
    tosys=1, height=12, justify="CH";
  plt, "Proterozoic", (lages(2)+lages(3))/2., 1.445, 
    tosys=1, height=12, justify="CH";
  plt, "Phanerozoic", (lages(3)+lages(4))/2., 1.445, 
    tosys=1, height=12, justify="CH";
  //  plt, "Future", (lages(4)+lages(5))/2., 1.4, 
  //    tosys=1, height=12, justify="CH";

  /*
  ages = [2.5,1.6,1.0,0.542];
  // convert this into relative solar luminosities:
  lages = gough_age_to_L( ages );
  pldj, lages, array( 0.5, 4 ), lages, array( 1., 4 );
  plg, [0.5,0.5], [lages(1),lages(0)];
  plt, "Paleoproterozoic", (lages(1)+lages(2))/2., 0.72, 
    tosys=1, height=12, justify="CH";
  plt, "Mesoprot.", (lages(2)+lages(3))/2., 0.72, 
    tosys=1, height=12, justify="CH";
  plt, "Neoprot.", (lages(3)+lages(4))/2., 0.72, 
    tosys=1, height=12, justify="CH", font="helveticaB";
  */

  // PLOT LIMITS AND TITLES: 
  logxy, 0, 1;
  limits, xl, xr, 1.e-8, 1.99;
  xytitles, "Solar luminosity relative to today", 
    "CO_2_ partial pressure  (bar)",
    [-0.02,0.01];
  topxtitle, "Time before present (Ga)", offset=0.04;
  // switch off ticks on top of the plot:
  gridxy, 0x621, 0;


  // PLOT BUILD OUTPUT:
  if( !is_void(builds) ){
    filename = "Plots/glac_figure1_build0";
    pdf, filename;
    png, filename, dpi=600;
  }

  
  // Snowball Earth events:
  s = array( 0, [2,1,1] );
  // Paleoproterozoic:
  pli, bytscl( s, cmin=0, cmax=1),
    gough_age_to_L(2.4), 1.e-8, gough_age_to_L(2.1), 1.;
  // Neoproterozoic - Sturtian:
  pli, bytscl( s, cmin=0, cmax=1),
    gough_age_to_L(0.717), 1.e-8, gough_age_to_L(0.662), 1.;
  // Neoproterozoic - Marinoan:
  pli, bytscl( s, cmin=0, cmax=1),
    gough_age_to_L(0.645), 1.e-8, gough_age_to_L(0.630), 1.;
  // plot this line again for cosmetic reasons...:
  plg, [1.,1.], [xl,xr];

  // create a special colour palette for this plot (light blue as
  // lowest value, light cyan as highest value, dark green in between)
  // - from the CCN plot in neop.i:
  r = [200]; grow, r, span(50, 255, 238); grow, r, [150];
  g = [230]; grow, g, span(75, 255, 238); grow, g, [255];
  b = [255]; grow, b, span(0, 255, 238); grow, b, [255];
  palette, r,g,b;


  // PLOT BUILD OUTPUT:
  if( !is_void(builds) ){
    filename = "Plots/glac_figure1_build1";
    pdf, filename;
    png, filename, dpi=600;
  }
  

  // PLOT CO2 PROXIES AND MEASUREMENTS:
  glac_figure1_proxies;

  
  // PLOT BUILD OUTPUT:
  if( !is_void(builds) ){
    filename = "Plots/glac_figure1_build2";
    pdf, filename;
    png, filename, dpi=600;
  }


  // PLOT MODEL RESULTS (but only in case of builds, otherwise we have
  // to do it twice):

  // PLOT BUILD OUTPUT:
  if( !is_void(builds) ){

    glac_figure1_models;

    plegend, [4,1,4,3],
      ["","AOGCM","OGCM","AGCM"],
      msize=[.7,.4,.4,.4], color=["white","black","black","black"],
      origin=[.075,.275];

    filename = "Plots/glac_figure1_build3";
    pdf, filename;
    png, filename, dpi=600;
    
  }

 
  // plot fit to aquaplanet simulations:
  if( is_void( builds ) )
    plot_fit_aqua;
  
  // plot aquaplanet simulations:
  read_columns, "ModelOutput/snowball_bifurcation_aquaplanet_final.dat",
    1,a_aqua, 2,l_aqua, 3,c_aqua_low, 4,c_aqua_high;
  // compute most likely critical CO2:
  // standard experiments with reduced CO2 at fixed solar luminosities:
  c_aqua = .5*(c_aqua_low+c_aqua_high);
  plmk, c_aqua/1.e6, l_aqua/1361., marker=4, msize=.7, color="blue";
  pldj,  l_aqua/1361., c_aqua_low/1.e6,  l_aqua/1361., c_aqua_high/1.e6,
    width=3, color="blue";
  pldj,  l_aqua/1361.-dl, c_aqua_low/1.e6,  l_aqua/1361.+dl, c_aqua_low/1.e6,
    width=3, color="blue";
  pldj,  l_aqua/1361.-dl, c_aqua_high/1.e6,  l_aqua/1361.+dl, c_aqua_high/1.e6,
    width=3, color="blue";
  // additional experiments at fixed CO2 levels with reduced solar constant:
  plmk, [10000.,10.]/1.e6, [1122.5,1326.5]/1361., marker=4, msize=.7, width=1,
    color="black";
  plmk, [10000.,10.]/1.e6, [1122.5,1326.5]/1361., marker=4, msize=.6, width=2,
    color="blue";
  //  plmk, [10000.,10.]/1.e6, [1122.5,1326.5]/1361., marker=4, msize=.53, width=1,
  //    color="black"

  glac_figure1_models;

  /*
  plegend, [4,1,4,3],
    ["This work","AOGCM","OGCM","AGCM"],
    msize=[.7,.4,.4,.4], color=["blue","black","black","black"],
    origin=[.075,.275];
  */
  plegend, [4,1,4,3,3],
    ["This work","AOGCM","OGCM","AGCM","AGCM w/o OHT"],
    msize=[.7,.4,.4,.4,.4], color=["blue","black","black","black","black"],
    width=[10,10,10,10,2], origin=[.075,.325];

  
  // PLOT BUILD OUTPUT:
  if( !is_void(builds) ){
    glac_figure1_models;
    filename = "Plots/glac_figure1_build4";
    pdf, filename;
    png, filename, dpi=600;
  }


  filename = "Plots/glac_figure1";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;
  
}


func plot_fit_aqua( void )
{

  p = comp_fit_aqua();
  p;
  x=span(0.75,1.,1000);
  plg, exp(_thresh_func_(x, p(,1)))/1.e6, x, width=3, color="red";


}

func comp_fit_aqua( void )
{
  
  // read data from aquaplanet simulations:
  read_columns, "ModelOutput/snowball_bifurcation_aquaplanet_final.dat",
    1,a_aqua, 2,l_aqua, 3,c_aqua_low, 4,c_aqua_high;
  // compute most likely critical CO2:
  c_aqua = .5*(c_aqua_low+c_aqua_high);

  // scale critical CO2 concentrations up to avoid problems with
  // floating point exceptions when the argument of the logarithm gets
  // negative:
  c_aqua *= 100.;
  
  // the fit will be done on the logarithms of the threshold values
  // and in solar luminosity relative to today:
  lc_aqua = log(c_aqua);
  x_aqua = l_aqua/1361.;

  // compute weights:       // this leads to problems in the fitting...!
  //  w = log(c_aqua/(c_aqua_high-c_aqua_low));
  //  www = where( w<0. );
  //  if( numberof(www) )
  //    w(www) = 0.;
  w = array( 1., numberof(x_aqua) );
  // exclude point at 0 Ma:
  //  w(1) = 0;             // fit does not converge once this point is excluded!
  
  // parameter guess (from eye-ball fit):
  //  p = [1.1,27.5,0.625];
  p = [0.41,27.4,1.0,0.46];

  // scale p(1) up because of the scaling above:
  p(1) += log(100.);
  
  // make the fit:
  f = lmfit2( _thresh_func_, x_aqua, p, lc_aqua, w, stdev=1 );

  // scale fit parameters down again:
  p(1) /= 100.;

  // get standard deviation for rescaling:
  s = *f.stdev;
  s(1) /= 100.;
  
  return [p,s];
  
}


func _thresh_func_( x, p )
{
  //  return log(p(1)) + p(2)*(1.-x)^p(3);
  f = x*0.;
  www = where( x<p(3) );
  f(www) = log(p(1)) + p(2)*(p(3)-x(www))^p(4);
  return f;
}




func glac_figure1_proxies( void )
{
  // Rye et al. (1995) upper limit:
  // 10^-1.4 atm = 0.0398 atm between 2.75 and 2.2 Ga 
  plg, .0398*[1,1], gough_age_to_L([2.75,2.2]), width=2, color=pcol;
  Lmid = (gough_age_to_L(2.75)+gough_age_to_L(2.2))/2.;
  plg, .0398*[.5,1], Lmid*[1.,1.], width=2, color=pcol;
  plg, .0398*[.5,.6], Lmid+[0.,dl], width=2, color=pcol;
  plg, .0398*[.6,.5], Lmid+[-dl,0.], width=2, color=pcol;
  plt, "R+95", Lmid, 0.0398*.375, height=8, tosys=1, color=pcol, justify="CH";
  

  // Hessler et al. (2004) lower limits:
  // age: 3.2 Ga corresponding to L/L0 = 0.78
  // lower limit at 25 deg C: 0.0025 bar
  plg, [0.0025,0.0025], 0.78+[-dl,dl], width=2, color=pcol; 
  plg, [0.0025,0.008], [0.78,0.78], width=2, color=pcol;
  plg, 0.008*[0.8,1.], 0.78+[-dl,0.], width=2, color=pcol;
  plg, 0.008*[0.8,1.], 0.78+[dl,0.], width=2, color=pcol;
  plt, "H04", 0.776, 0.0015, height=8, tosys=1, color=pcol;

  // Sheldon (2006) measurement:
  // age: 2.2 Ga corresponding to L/L0 = 0.84 
  // range: (23 /3 *3) PAL "PAL" is not defined, at some point in
  // the paper there is the conversion from 2.51e-3 bar to 6.8 PAL,
  // from which one can conclude that PAL = 370ppm:
  local a, c;
  read_columns, "../ArcheanClimate/Data/sheldon_2006.dat", 1,a, 2,c;
  l = gough_age_to_L( a );
  c *= 0.00037;
  for( n=1; n<=numberof(c); n++ ){
    plg, [c(n)/3.,c(n)*3.], l(n)+[0,0], width=2, color=pcol;
    plg, [c(n)/3.,c(n)/3.], l(n)+[-dl,dl], width=2, color=pcol;
    plg, [c(n)*3.,c(n)*3.], l(n)+[-dl,dl], width=2, color=pcol;
    plt, "S06", l(n), c(n)/4.5, height=8, tosys=1, color=pcol, justify="CH";
  }

  // Rosing et al. (2010) upper limit:
  // the upper limit of 0.0009 bar is for 25 deg C
  plg, 0.0009*[1.,1.], [0.775,0.833], width=2, color=pcol;
  plg, 0.0009*[1.,0.5], [0.804,0.804], width=2, color=pcol;
  plg, 0.0009*[0.5,0.6], 0.804+[0.,-dl], width=2, color=pcol;
  plg, 0.0009*[0.5,0.6], 0.804+[0.,dl], width=2, color=pcol;
  plt, "R+10", 0.804+0.002, 0.0009*.35, height=8, tosys=1, color=pcol, justify="CH";

  // Driese et al. (2011) measurement:
  // age: 2.69 Ga, corresponding to L/L0 = 0.813
  // best estimate: 41 times PAL
  // range: 10-50 times PAL; "PAL" is not defined, at some point in 
  // the paper there is the conversion from 0.009 bar to 25 PAL,
  // from which one can derive a value of PAL = 360ppm:
  //  plmk, 41.*0.00036, 0.813, marker=4, msize=.5, color=[255,165,0];
  plg, [10.,50.]*0.00036, 0.813+[0.,0.], width=2, color=pcol;
  plg, [10.,10.]*0.00036, 0.813+[-dl,dl], width=2, color=pcol;
  plg, [50.,50.]*0.00036, 0.813+[-dl,dl], width=2, color=pcol;
  plt, "D+11", 0.808, 0.0025, height=8, tosys=1, color=pcol;

  // Kanzaki & Murakami (2015) paleosol reconstruction:
  // Again, "PAL" is not defined in the paper, from Fig. 9 one can
  // deduce that PAL = 320ppm:
  // 2.77 Ga:
  plg, [85.,510.]*0.00032, gough_age_to_L(2.77)*[1,1], width=2, color=pcol;
  plg, [85.,85.]*0.00032, gough_age_to_L(2.77)+[-dl,+dl], width=2, color=pcol;
  plg, [85.,85.]*0.00032, gough_age_to_L(2.77)+[-dl,+dl], width=2, color=pcol;
  plg, [510.,510.]*0.00032, gough_age_to_L(2.77)+[-dl,+dl], width=2, color=pcol;
  plg, [510.,510.]*0.00032, gough_age_to_L(2.77)+[-dl,+dl], width=2, color=pcol;
  plt, "KM15",  gough_age_to_L(2.77), 510.*0.00032*1.5, height=8, tosys=1, color=pcol, justify="CH";
  // 2.75 Ga:
  plg, [78.,2500.]*0.00032, gough_age_to_L(2.75)*[1,1], width=2, color=pcol;
  plg, [78.,78.]*0.00032, gough_age_to_L(2.75)+[-dl,+dl], width=2, color=pcol;
  plg, [78.,78.]*0.00032, gough_age_to_L(2.75)+[-dl,+dl], width=2, color=pcol;
  plg, [2500.,2500.]*0.00032, gough_age_to_L(2.75)+[-dl,+dl], width=2, color=pcol;
  plg, [2500.,2500.]*0.00032, gough_age_to_L(2.75)+[-dl,+dl], width=2, color=pcol;
  //  plt, "KM15",  gough_age_to_L(2.55), 2500.*0.00032*.9, height=8, tosys=1, color=pcol, justify="CH";
  // 2.46 Ga:
  plg, [160.,490.]*0.00032, gough_age_to_L(2.46)*[1,1], width=2, color=pcol;
  plg, [160.,160.]*0.00032, gough_age_to_L(2.46)+[-dl,+dl], width=2, color=pcol;
  plg, [160.,160.]*0.00032, gough_age_to_L(2.46)+[-dl,+dl], width=2, color=pcol;
  plg, [490.,490.]*0.00032, gough_age_to_L(2.46)+[-dl,+dl], width=2, color=pcol;
  plg, [490.,490.]*0.00032, gough_age_to_L(2.46)+[-dl,+dl], width=2, color=pcol;
  plt, "KM15",  gough_age_to_L(2.46), 490.*0.00032*1.5, height=8, tosys=1, color=pcol, justify="CH";
  // 2.15 Ga:
  plg, [30.,190.]*0.00032, gough_age_to_L(2.15)*[1,1], width=2, color=pcol;
  plg, [30.,30.]*0.00032, gough_age_to_L(2.15)+[-dl,+dl], width=2, color=pcol;
  plg, [30.,30.]*0.00032, gough_age_to_L(2.15)+[-dl,+dl], width=2, color=pcol;
  plg, [190.,190.]*0.00032, gough_age_to_L(2.15)+[-dl,+dl], width=2, color=pcol;
  plg, [190.,190.]*0.00032, gough_age_to_L(2.15)+[-dl,+dl], width=2, color=pcol;
  plt, "KM15",  gough_age_to_L(2.15), 190.*0.00032*1.5, height=8, tosys=1, color=pcol, justify="CH";
  // 2.08 Ga:
  plg, [20.,620.]*0.00032, gough_age_to_L(2.08)*[1,1], width=2, color=pcol;
  plg, [20.,20.]*0.00032, gough_age_to_L(2.08)+[-dl,+dl], width=2, color=pcol;
  plg, [20.,20.]*0.00032, gough_age_to_L(2.08)+[-dl,+dl], width=2, color=pcol;
  plg, [620.,620.]*0.00032, gough_age_to_L(2.08)+[-dl,+dl], width=2, color=pcol;
  plg, [620.,620.]*0.00032, gough_age_to_L(2.08)+[-dl,+dl], width=2, color=pcol;
  plt, "KM15",  gough_age_to_L(2.08), 620.*0.00032*1.5, height=8, tosys=1, color=pcol, justify="CH";
  // 1.85 Ga:
  plg, [23.,210.]*0.00032, gough_age_to_L(1.85)*[1,1], width=2, color=pcol;
  plg, [23.,23.]*0.00032, gough_age_to_L(1.85)+[-dl,+dl], width=2, color=pcol;
  plg, [23.,23.]*0.00032, gough_age_to_L(1.85)+[-dl,+dl], width=2, color=pcol;
  plg, [210.,210.]*0.00032, gough_age_to_L(1.85)+[-dl,+dl], width=2, color=pcol;
  plg, [210.,210.]*0.00032, gough_age_to_L(1.85)+[-dl,+dl], width=2, color=pcol;
  plt, "KM15",  gough_age_to_L(1.85), 210.*0.00032*1.5, height=8, tosys=1, color=pcol, justify="CH";
                                                                    

  // Strauss & Tosca (2020) Neoproterozoic marine carbonate chemistry
  // with two CO2 ranges: 872-1750 ppm at 805-788 Ma (796.5 Ma), 165-724 ppm at
  // 755-735 Ma (745 Ma):
  plg, [872,1750]/1.e6, gough_age_to_L(0.7965)*[1,1], width=2, color=pcol;
  plg, [872,872]/1.e6, gough_age_to_L(0.7965)+[-dl,dl], width=2, color=pcol;
  plg, [1750,1750]/1.e6, gough_age_to_L(0.7965)+[-dl,dl], width=2, color=pcol;
  plg, [165, 724]/1.e6, gough_age_to_L(0.7450)*[1,1], width=2, color=pcol;
  plg, [165,165]/1.e6, gough_age_to_L(0.7450)+[-dl,dl], width=2, color=pcol;
  plg, [724,724]/1.e6, gough_age_to_L(0.7450)+[-dl,dl], width=2, color=pcol;
  plt, "ST20",  gough_age_to_L(0.77075), 3.e-3, height=8, tosys=1, color=pcol, justify="CH";

  // Lehmer et al. (2020) inferred from 2.7 Ga micrometeorites; volume
  // mixing ratio 0.64 (-0.58, +0.36, 2 sigma); for a total pressure
  // of 1 bar this translates into 0.06-1.0 bar
  plg, [0.06,1.0], gough_age_to_L(2.7)*[1,1], width=2, color=pcol;
  plg, [0.06,0.06], gough_age_to_L(2.7)+[-dl,dl], width=2, color=pcol;
  plg, [1.,1.], gough_age_to_L(2.7)+[-dl,dl], width=2, color=pcol;
  plt, "L+20",  gough_age_to_L(2.55), .6, height=8, tosys=1, color=pcol, justify="CH";
  
  /*
  // Royer (2006) compilation:
  read_columns, "~/Climate/Data/royer2006co2.dat",
    1,aa, 4,cc;
  // throw away upper limits:
  nonzero = where( cc>0. );
  cc = cc(nonzero); aa = aa(nonzero);
  ss = sort(aa); // range: 0.1 Ma to 447 Ma
  cc = cc(ss); aa = aa(ss);
  plmk, cc/1.e6, gough_age_to_L( aa/1000. ), marker=1, msize=.1, color=pcol;
  //  plg, cc/1.e6, gough_age_to_L( aa/1000. ), width=1, color=pcol;
  // OK, we need a binned version of this: interp() does not work, 10
  // Myr intervals aren't great either; let's try an n-point running
  // mean:
  np = 9;
  ns = int(floor(np/2.));
  aaa = ccc = [];
  for( n=ns+1; n<numberof(aa)-ns; n+=1 ){
    grow, ccc, avg(cc(n-ns:n+ns));
    grow, aaa, avg(aa(n-ns:n+ns));
  }
  plg, ccc/1.e6, gough_age_to_L( aaa/1000. ), width=2, color=pcol;
  */

  // Foster et al. (2017) compilation:
  read_columns, "~/Climate/Data/Foster2017_allData.dat",
    1,aa, 2,cc;
  plmk, cc/1.e6, gough_age_to_L( aa/1000. ), marker=5, msize=.1, color=[200,200,200];
  read_columns, "~/Climate/Data/Foster2017_Fit2.dat",
    1,aa, 2,cc, 3,cc_low, 4,cc_high;
  plg, cc/1.e6, gough_age_to_L( aa/1000. ), width=2, color=pcol;
  //  plg, cc_low/1.e6, gough_age_to_L( aa/1000. ), width=1, color=pcol;
  //  plg, cc_high/1.e6, gough_age_to_L( aa/1000. ), width=1, color=pcol;
  
  // modern value:
  plmk, 415./1.e6, 1., marker=5, msize=.4, color=pcol;
  //  plmk, 400./1.e6, 1., marker=1, msize=.3, color=pcol;

}


func glac_figure1_models( void )
{
  // plot selected 3D model results for critical CO2:

  // ARCHEAN:

  // Kienert et al. (2012):
  // compute partial pressure, the quoted values are surface pressures
  // in the absence of any other gas; note that p_IN2 = 0.8 bar:
  p_ICO2 = [0.4,0.25,0.13];
  //  p_CO2  = (p_ICO2/44.)/(p_ICO2/44.+0.8/28.)*(0.8+p_ICO2);
  p_CO2  = p_ICO2/(0.8+p_ICO2);
  plmk, p_CO2, [0.7525,0.79,0.82], marker=4, msize=.4;
  plt, "K+12", 0.7525, p_CO2(1)/1.75, height=8, tosys=1, justify="CH";
  plt, "K+12", 0.7900, p_CO2(2)/1.75, height=8, tosys=1, justify="CH";
  plt, "K+12", 0.8200, p_CO2(3)/1.75, height=8, tosys=1, justify="CH";

  // Charnay et al. (2013), no methane, modern cloud droplet size:
  // S0 = 1366 W/m2; solar constant values as given in the paper
  // mean surface pressure 1 bar, partial pressures as reported
  // in the paper
  // double-checked: 17.9.2013, corrected: 10.11.2020
  // 3.8 Ga:
  plg, [0.0009,0.01], 1025.1/1361.*[1.,1.], width=2, color="black";
  plg, 0.0009*[1,1], 1025.1/1361.+[-dl,+dl], width=2, color="black";
  plg, 0.01*[1,1], 1025.1/1361.+[-dl,+dl], width=2, color="black";
  plmk, .5*(0.0009+0.01), 1025.1/1361., marker=3, msize=.4, color="black";
  plt, "C+13", 1025.1/1361., 0.0009/1.5, height=8, tosys=1, justify="CH";
  // 3.0 Ga:
  plmk, 0.0009, 1081.9/1361., marker=3, msize=.4, color="black";
  plg, 0.0009*[1.,.5],     1081.9/1361.*[1.,1.], width=2, color="black";
  plg, 0.0009*.5*[1.2,1.], 1081.9/1361.+[-dl,0.], width=2, color="black";
  plg, 0.0009*.5*[1.2,1.], 1081.9/1361.+[+dl,0.], width=2, color="black";
  plt, "C+13", 1081.9/1361.-.002, 0.0009*.35, height=8, tosys=1, justify="CH";
  // 2.5 Ga:
  plmk, 0.0009, 1120.8/1361+0.0005, marker=3, msize=.4, color="black";
  plg, 0.0009*[1.,.5],     (1120.8/1361.+0.0005)*[1.,1.], width=2, color="black";
  plg, 0.0009*.5*[1.2,1.], (1120.8/1361.+0.0005)+[-dl,0.], width=2, color="black";
  plg, 0.0009*.5*[1.2,1.], (1120.8/1361.+0.0005)+[+dl,0.], width=2, color="black";
  plt, "C+13", 1120.8/1361.+0.0005, 0.0009*.35, height=8, tosys=1, justify="LH";

  
  // Wolf & Toon (2013):
  // S0: solar constant value as given in the paper
  // total pressure 1.013 bar held constant
  // published in the paper
  // BC: present-day continents and rotation rate
  // double-checked: 17.9.2013, corrected: 28.10.2020
  plmk, 5.e-3*1.013, 1093.6/1361., marker=3, msize=.4, color="black";
  plg, 5.e-3*1.013*[1.,.5],     1093.6/1361.*[1.,1.], width=2, color="black";
  plg, 5.e-3*1.013*.5*[1.2,1.], 1093.6/1361.+[-dl,0.], width=2, color="black";
  plg, 5.e-3*1.013*.5*[1.2,1.], 1093.6/1361.+[+dl,0.], width=2, color="black";
  plt, "WT13", 1093.6/1361., 0.0015, height=8, tosys=1, justify="CH";

  // Kunze et al. (2014):
  // S0: 1365 W/m2; solar constant values as given in the paper
  // BC: aquaplanet, present-day rotation rate, spectral irradiance
  //     from solar analogue, no information on CH4, assuming 0 ppb
  // total pressure 1 bar in all simulations (M. Kunze, priv. comm.)
  // double-checked: 17.9.2013, corrected: 28.10.2020
  plmk, 3670.*1.e-6/1., 1046./1361., marker=3, msize=.4, color="black";
  plg, 3670.*1.e-6/1.*[1.,.5], 1046./1361.*[1.,1.], width=2, color="black";
  plg, 3670.*1.e-6/1.*.5*[1.2,1.], 1046./1361.+[-dl,0.], width=2, color="black";
  plg, 3670.*1.e-6/1.*.5*[1.2,1.], 1046./1361.+[+dl,0.], width=2, color="black";
  plmk, (3670.+367.)/2.*1.e-6/1., 1121./1361.-0.0005, marker=3, msize=.4, color="black";
  plg, [367.,3670.]*1.e-6/1., (1121./1361.-0.0005)*[1.,1.], width=2, color="black";
  plg, 367.0/1.e6*[1,1], (1121./1361.-0.0005)+[-dl,+dl], width=2, color="black";
  plg, 3670./1.e6*[1,1], (1121./1361.-0.0005)+[-dl,+dl], width=2, color="black";
  plt, "K+14", 1046./1361., 0.0012, height=8, tosys=1, justify="CH";
  plt, "K+14", 1121./1361.-0.0005, 0.00023, height=8, tosys=1, justify="RH";

    // Le Hir et al. (2013), modern cloud droplet size, 1.7e-6 CH4:
  // BC: realistic topography
  // S0 = ?;  solar constant values as given in the paper
  // 3.5 Ga, pCH4=1.7e-6 bar = 1700 ppb, pCO2=0.0252 bar (snowball), 0.0560 bar (warm)
  // double-checked: 17.9.2013, corrected: 10.11.2020
  // total forcing:
  forc = co2_c2f_old( [25200.,56000.] ) + ch4_c2f_old( 1700. );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plg, co2e/1.e6, 1053./1361.*[1,1], width=3, color="white";
  plg, co2e(1)/1.e6*[1,1], 1053./1361.+[-dl,+dl], width=3, color="white";
  plg, co2e(2)/1.e6*[1,1], 1053./1361.+[-dl,+dl], width=3, color="white";
  plmk, co2e(avg)/1.e6, 1053./1361., marker=3, msize=.45, color="white";
  plg, co2e/1.e6, 1053./1361.*[1,1], width=2, color="black";
  plg, co2e(1)/1.e6*[1,1], 1053./1361.+[-dl,+dl], width=2, color="black";
  plg, co2e(2)/1.e6*[1,1], 1053./1361.+[-dl,+dl], width=2, color="black";
  plmk, co2e(avg)/1.e6, 1053./1361., marker=3, msize=.4, color="black";
  plt, "L+14", 1053./1361., 1.5e-2, height=8, tosys=1, justify="CH";


  // PALEOPROTEROZOIC:
  
  // Teitler et al. (2014): AGCM, mixed-layer ocean, diffusive heat transport,
  // no sea-ice dynamics
  // BC: solar constant 1100 W/m2, reconstructed paleogeography, 18 hours daylength,
  //     higher salinity (freezing point at -3.3 deg. C), 1 bar total pressure
  // open water at 5.5e-3 bar CO2 + 60 ppmv CH4
  // snowball   at 4.0e-3 bar CO2 + 60 ppmv CH4
  // total forcing:
  forc = co2_c2f_old( [4000.,5500.] ) + ch4_c2f_old( 60000. );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  
  plmk, avg(co2e)/1.e6, 1100./1361., marker=3, msize=.45, color="white";
  plg, co2e/1.e6, 1100./1361.*[1,1], width=3, color="white";
  plg, co2e(1)/1.e6*[1,1], 1100./1361.+[-dl,+dl], width=3, color="white";
  plg, co2e(2)/1.e6*[1,1], 1100./1361.+[-dl,+dl], width=3, color="white";
  plmk, avg(co2e)/1.e6, 1100./1361., marker=3, msize=.4, color="black";
  plg, co2e/1.e6, 1100./1361.*[1,1], width=2, color="black";
  plg, co2e(1)/1.e6*[1,1], 1100./1361.+[-dl,+dl], width=2, color="black";
  plg, co2e(2)/1.e6*[1,1], 1100./1361.+[-dl,+dl], width=2, color="black";
  plt, "T+14", 1100./1361., 0.009, height=8, tosys=1, justify="CH";


  // MESOROTEROZOIC:

  // Fiorella & Sheldon (2017):
  // AOGCM (CESM), 1.0 Ga
  // S0 "reduced by 9%", present-day S0 1367 W/m2, value used in the simulations
  // is 1244 W/m2 (R. Fiorella, priv. comm.)
  // without CH4, snowball at 280 ppm CO2, no snowball at 1400 ppm
  plg, [280.,1400.]/1.e6, 1244.*[1,1]/1361., width=2, color="black";
  plg, 280.0/1.e6*[1,1],  1244./1361.+[-dl,+dl], width=2, color="black";
  plg, 1400./1.e6*[1,1],  1244./1361.+[-dl,+dl], width=2, color="black";
  plmk, 0.5*(280.+1440.)/1.e6, 1244./1361., marker=1, msize=.4, color="black";
  //  plt, "FS17", 1244./1361., 0.0025, height=8, tosys=1, justify="CH";
  plt, "FS17", 0.902, 0.0019, height=8, tosys=1, justify="CH";
  plg, [0.0014,0.001], [0.903,0.911], width=1;


  
  // NEOPROTEROZOIC:

  // Chandler & Sohl (2000): AGCM, solar constant 1285 W/m2,
  // continental configuration for 600 Ma, snowball bifurcation below
  // 40 ppm:
  plg, 40./1.e6*[1.,.5],     1285./1361.*[1.,1.], width=3, color="white";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[-dl,0.], width=3, color="white";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[+dl,0.], width=3, color="white";
  plmk, 40./1.e6, 1285./1361., marker=3, msize=.4, color="black";
  plg, 40./1.e6*[1.,.5],     1285./1361.*[1.,1.], width=2, color="black";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[-dl,0.], width=2, color="black";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[+dl,0.], width=2, color="black";
  plt, "CS00", 0.935, 1.e-5, height=8, tosys=1, justify="RH";
  //  plt, "CS00", 1285./1361., 1.5e-5, height=8, tosys=1, justify="CH";
  plg, [1.3e-5,3.4e-5], [0.935,0.941], width=1;
  
  // Poulsen et al. (2002): AOGCM (FOAM), solar constant 93% of
  // (presumably) 1367 W/m2 (this is the value in the standard FOAM
  // atmos_params file), continental configuration for 545 Ma,
  // simulations with 140 ppm of CO2 and 1714 ppb of CH4, no
  // glaciation, so this is an upper limit:  
  // total forcing:
  forc = co2_c2f_old( 140. ) + ch4_c2f_old( 1714. );  
  // CO2 equivalent:
  co2e = co2_f2c( forc );
  plmk, co2e/1.e6, 0.93*1367./1361., marker=1, msize=.4, color="black";
  plg, co2e/1.e6*[1.,.5],     0.93*1367./1361.*[1.,1.], width=2, color="black";
  plg, co2e/1.e6*.5*[1.2,1.], 0.93*1367./1361.+[-dl,0.], width=2, color="black";
  plg, co2e/1.e6*.5*[1.2,1.], 0.93*1367./1361.+[+dl,0.], width=2, color="black";
  plt, "P+02", 0.925*1367./1361., 5.5e-5, height=8, tosys=1, justify="CH";
  plg, [7.e-5,1.5e-4], [0.925,0.9285]*1367./1361., width=1;

  // Lewis et al. (2003): UVic (OGCM), solar constant reduced by 6%,
  // present-day value not specified, but 1368 W/m2 according to UVic
  // description paper; uniform geothermal heat flux; faster rotation
  // rate (18h day length); different idealised supercontinents
  // (equatorial, pole-to-pole, SH). Bifurcation between about 1500
  // ppm and 2000 ppm depending on continental configuration:
  plmk, 1750./1.e6, 0.94*1368./1361., marker=4, msize=.4, color="black";
  plg, [1500,2000]/1.e6, 0.94*1368./1361.*[1,1], width=2, color="black";
  plg, [1500,1500]/1.e6, 0.94*1368./1361.+[-dl,dl], width=2, color="black";
  plg, [2000,2000]/1.e6, 0.94*1368./1361.+[-dl,dl], width=2, color="black";
  plt, "L+03", 0.951, 1750./1.e6, height=8, tosys=1, justify="LH";

  // Pierrehumbert et al. (2011): Three AGCMs plus mixed-layer ocean
  // without ocean heat transport; solar constant at 1285 W/m2,
  // non-CO2 greenhouse gases treated differently in the three models,
  // so ignored here; critical states are between 2000 and 6000 ppm
  // for the standard albedos:
  sol = 1284.;
  plg, [2000.,6000.]/1.e6, sol/1361.*[1,1], width=2;
  plg, [2000,2000]/1.e6, sol/1361.+[-dl,dl], width=2, color="black";
  plg, [6000,6000]/1.e6, sol/1361.+[-dl,dl], width=2, color="black";
  plmk, 4000./1.e6, sol/1361., marker=3, msize=.4, width=2, color="black";
  plt, "P+11", 0.93, 4000.*1.2/1.e6, height=8, tosys=1, justify="LH";
  //  plg, 4000./1.e6*[1.1,1.2], [0.947,0.951], width=1;
  
  // Voigt et al. (2011): AOGCM, solar constant 1285 W/m2, Marinoan,
  // CH4 at 650 ppb, N2O at 270 ppb, bifurcation somewhere between 286
  // and 572 ppm:
  // total forcing:
  forc1 = co2_c2f_old( 286. ) + ch4_c2f_old( 650. );
  forc2 = co2_c2f_old( 572. ) + ch4_c2f_old( 650. );
  // CO2 equivalent:
  co2e1 = co2_f2c_old( forc1 );
  co2e2 = co2_f2c_old( forc2 );
  co2e = (co2e1+co2e2)/2.;
  plmk, co2e/1.e6, 1285./1361., marker=1, msize=.4, color="black";
  plg, [co2e1,co2e2]/1.e6, [1285./1361.,1285./1361.], width=2, color="black";
  plg, co2e1/1.e6*[1,1], 1285./1361.+[-dl,+dl], width=2, color="black";
  plg, co2e2/1.e6*[1,1], 1285./1361.+[-dl,+dl], width=2, color="black";
  //  plt, "V+11", 1285./1361., 7.e-4, height=8, tosys=1, justify="CH";
  plt, "V+11", 0.9525, 1.4*co2e/1.e6, height=8, tosys=1, justify="LH";
  plg, co2e/1.e6*[1,1.2], [0.947,0.951], width=1;
  plg, co2e/1.e6*[.85,1.05], [0.961,0.959], width=1;

  // Voigt & Abbot (2012): AOGCM (ECHAM5/MPI-OM), lower sea-ice albedo
  // compared to Voigt & Marotzke (2010); solar constant 1285 W/m2,
  // Marinoan, CH4 at 650 ppb, N2O at 270 ppb, bifurcation somewhere
  // between 204 and 209 ppm:
  // total forcing:
  forc1 = co2_c2f_old( 204. ) + ch4_c2f_old( 650. );
  forc2 = co2_c2f_old( 209. ) + ch4_c2f_old( 650. );
  // CO2 equivalent:
  co2e1 = co2_f2c_old( forc1 );
  co2e2 = co2_f2c_old( forc2 );
  co2e = (co2e1+co2e2)/2.;
  plmk, co2e/1.e6, 1285./1361., marker=1, msize=.4, color="black";
  plg, [co2e1,co2e2]/1.e6, [1285./1361.,1285./1361.], width=2, color="black";
  plt, "VA12", 0.9525, 1.0*co2e/1.e6, height=8, tosys=1, justify="LH";
  plg, co2e/1.e6*[1,1], [0.947,0.951], width=1;


  // Yang et al. (2012a): AOGCM (CCSM3), modern snowballs, solar
  // constant 94% of 1367 W/m2 (so basically Neoproterozoic), CH4 at
  // 805.6 ppb, N2O at 276.7 ppb, snowball bifurcation between 20 ppm
  // and 17.5 ppm:  
  // total forcing:
  forc = co2_c2f_old( 18.75 ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plmk, co2e/1.e6, .94*1367./1361., marker=1, msize=.4, color="black";
  plt, "Y+12a", .952*1367./1361., 1.4e-5, height=8, tosys=1, justify="CH";
  plg, [1.7e-5,1.5e-5], [0.947,0.949], width=1;
  // re-plot CS00 arrow:
  plg, 40./1.e6*[1.,.5],     1285./1361.*[1.,1.], width=3, color="white";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[-dl,0.], width=3, color="white";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[+dl,0.], width=3, color="white";
  plmk, 40./1.e6, 1285./1361., marker=3, msize=.4, color="black";
  plg, 40./1.e6*[1.,.5],     1285./1361.*[1.,1.], width=2, color="black";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[-dl,0.], width=2, color="black";
  plg, 40./1.e6*.5*[1.2,1.], 1285./1361.+[+dl,0.], width=2, color="black";

  // Yang et al. (2012c): AOGCM (CCSM4), modern snowballs, solar
  // constant 94% of 1367 W/m2 (so basically Neoproterozoic), CH4 at
  // 805.6 ppb, N2O at 276.7 ppb, snowball bifurcation between 100 ppm
  // and 70 ppm:
  // total forcing:
  forc = co2_c2f_old( 85. ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plmk, co2e/1.e6, .94*1367./1361., marker=1, msize=.45, color="white";
  plmk, co2e/1.e6, .94*1367./1361., marker=1, msize=.4, color="black";
  plg, [70.,100.]/1.e6, .94*1367./1361.*[1,1], width=3, color="white";
  plg, 70./1.e6*[1,1], .94*1367./1361.+[-dl,+dl], width=3, color="white";
  plg, 100./1.e6*[1,1], .94*1367./1361.+[-dl,+dl], width=3, color="white";
  plg, [70.,100.]/1.e6, .94*1367./1361.*[1,1], width=2, color="black";
  plg, 70./1.e6*[1,1], .94*1367./1361.+[-dl,+dl], width=2, color="black";
  plg, 100./1.e6*[1,1], .94*1367./1361.+[-dl,+dl], width=2, color="black";
  plt, "Y+12c", 0.955, 8.e-5, height=8, tosys=1, justify="LH";
  plg, [co2e/1.e6,8.e-5], [.947,.954], width=1;

  // Liu et al. (2013): solar constant 0.94*1367 W/m2, CH4 at 805.6
  // ppb, N2O at 276.7 ppb, two continental configurations at the same
  // solar constant, bifurcation between 80-90 ppm and 140-150 ppm,
  // respectively:
  forc1 = co2_c2f_old( 80. ) + ch4_c2f_old( 805.6 );
  forc2 = co2_c2f_old( 150. ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e1 = co2_f2c_old( forc1 );
  co2e2 = co2_f2c_old( forc2 );
  plg, [co2e1,co2e2]/1.e6, [1,1]*1285./1361, width=3, color="white";
  plg, [co2e1,co2e1]/1.e6, 1285./1361.+[-dl,dl], width=3, color="white";
  plg, [co2e2,co2e2]/1.e6, 1285./1361.+[-dl,dl], width=3, color="white";
  plmk, (co2e1+co2e2)/2./1.e6, 1285./1361., marker=1, msize=.45, color="white";
  plmk, (co2e1+co2e2)/2./1.e6, 1285./1361., marker=1, msize=.4, color="black";
  plg, [co2e1,co2e2]/1.e6, [1,1]*1285./1361, width=2, color="black";
  plg, [co2e1,co2e1]/1.e6, 1285./1361.+[-dl,dl], width=2, color="black";
  plg, [co2e2,co2e2]/1.e6, 1285./1361.+[-dl,dl], width=2, color="black";
  //  plt, "L+13", 1285./1361., 6.5e-5, height=8, tosys=1, justify="CH";
  plt, "L+13", 0.955,  (co2e1+co2e2)/2./1.e6, height=8, tosys=1, justify="LH";
  plg,  (co2e1+co2e2)/2./1.e6*[1,1], [0.947,0.954], width=2, color="white";
  plg,  (co2e1+co2e2)/2./1.e6*[1,1], [0.947,0.954], width=1;

  // Feulner & Kienert (2014):
  plmk, 105./1.e6, 1283./1361., marker=4, msize=.45, color="white";
  plmk, 105./1.e6, 1283./1361., marker=4, msize=.4, color="black";
  plg, [100.,110.]/1.e6, [1,1]* 1283./1361., width=2;
  plmk, 125./1.e6, 1290./1361., marker=4, msize=.4, color="black";
  plg, [120.,130.]/1.e6, [1,1]* 1290./1361., width=2;
  //  plt, "FK14", (1283.+1290.)/(2.*1361.), 2.e-4, height=8, tosys=1, justify="CH";
  plt, "FK14", 0.93, 2.2e-5, height=8, tosys=1, justify="CH";
  plg, [3.e-5,7.e-5], [0.935,0.941], width=1;
  // re-plot line connecting label to data point from L+13:
  plg,  (co2e1+co2e2)/2./1.e6*[1,1], [0.947,0.954], width=2, color="white";
  plg,  (co2e1+co2e2)/2./1.e6*[1,1], [0.947,0.954], width=1;


  // Feulner, Hallmann & Kienert (2015):
  //  plmk,  15./1.e6, 1283./1361., marker=4, msize=.4, width=1;
  //  plmk,  15./1.e6, 1283./1361., marker=4, msize=.3, color=[150,255,255];
  //  plt, "F+15", 1283./1361., 9.e-6, height=8, tosys=1, justify="CH";
  
  // Liu et al. (2017): AOGCM (CCSM3) with ice-sheets, continental
  // configurations for 720 and 570 Ma, but same value for solar
  // constant, 94% of present-day value (1285 W/m2, explicitly given
  // in the paper), CH4 at 805.6 ppb, N2O at 276.7 ppb (as in Liu et
  // al. 2013), snowball bifurcation between 630 and 600 ppm for 720
  // Ma and between 320 and 300 ppm for 570 Ma:
  forc1 = co2_c2f_old( 300. ) +
    ch4_c2f_old( 805.6 );
  forc2 = co2_c2f_old( 630. ) +
    ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e1 = co2_f2c_old( forc1 );
  co2e2 = co2_f2c_old( forc2 );
  plg, [co2e1,co2e2]/1.e6, [1,1]*1285./1361, width=3, color="white";
  plg, [co2e1,co2e1]/1.e6, 1285./1361.+[-dl,dl], width=3, color="white";
  plg, [co2e2,co2e2]/1.e6, 1285./1361.+[-dl,dl], width=3, color="white";
  plmk, (co2e1+co2e2)/2./1.e6, 1285./1361., marker=1, msize=.45, color="white";
  plmk, (co2e1+co2e2)/2./1.e6, 1285./1361., marker=1, msize=.4, color="black";
  plg, [co2e1,co2e2]/1.e6, [1,1]*1285./1361, width=2, color="black";
  plg, [co2e1,co2e1]/1.e6, 1285./1361.+[-dl,dl], width=2, color="black";
  plg, [co2e2,co2e2]/1.e6, 1285./1361.+[-dl,dl], width=2, color="black";
  plg, (co2e1+co2e2)/2./1.e6*[1,1], [0.940,0.942], width=1;
  plt, "L+17", 0.94, (co2e1+co2e2)/2./1.e6, height=8, tosys=1, justify="RH";

  // Braun et al. (2022): Two AGCMs (CAM, ICON), mixed-layer ocean
  // without heat transport; 1285 W/m2, no non-CO2 greenhouse gases;
  // different cloud modules; range of 2000 to 5500 ppm representing
  // the different models and cloud schemes:
  co2 = (2000.+5500.)/2.;
  sol = 1286.;
  plg, [2000.,5500.]/1.e6, sol/1361.*[1,1], width=2;
  plg, [2000,2000]/1.e6, sol/1361.+[-dl,dl], width=2, color="black";
  plg, [5500,5500]/1.e6, sol/1361.+[-dl,dl], width=2, color="black";
  plmk, co2/1.e6, sol/1361., marker=3, msize=.4, width=2;
  plt, "B+22", 0.9525, co2*1.3/1.e6, height=8, tosys=1, justify="LH";
  plg, co2/1.e6*[1.1,1.3], [0.947,0.951], width=1;
  
  // Hörner et al. (2022): AGCM (ICON-A) plus mixed-layer ocean
  // without heat transport; aquaplanet configuration; testing two
  // different sea-ice models; 1285 W/m2, no non-CO2 greenhouse gases;
  // glaciation between 1547 and 2344 ppm depending on the sea-ice
  // scheme:
  co2 = (1547.+2344.)/2.;
  plg, [1547.,2344.]/1.e6, 1285./1361.*[1,1], width=2;
  plg, [1547,1547]/1.e6, 1285./1361.+[-dl,dl], width=2, color="black";
  plg, [2344,2344]/1.e6, 1285./1361.+[-dl,dl], width=2, color="black";
  plmk, co2/1.e6, 1285./1361., marker=3, msize=.4, width=2;
  plt, "H+22", 0.9525, co2*1.3/1.e6, height=8, tosys=1, justify="LH";
  plg, co2/1.e6*[1.1,1.3], [0.947,0.951], width=1;
  
  
  
  // CARBONIFEROUS/PERMIAN: Feulner (2017): solar constant 1327 W/m2,
  // bifurcation between 40 and 35 ppm:
  plmk, 37.5/1.e6, 1327./1361., marker=4, msize=.4, color="black";
  plg, [35,40]/1.e6, [1,1]*1327./1361, width=2, color="black";
  plt, "F17", 1327./1361., 2.1e-5, height=8, tosys=1, justify="CH";


  
  // MODERN:

  // Romanova et al. (2006): AGCM (PUMA), mixed-layer ocean,
  // thermodynamic sea-ice model, present-day continents, solar
  // constant 1367 W/m2 (not specified in the paper, from PlaSim
  // code), no snowball at 1 ppm CO2:
  plmk, 1.e-6, 1367./1361., marker=3, msize=.4, color="black";
  plg, 1.e-6*[1.,.5],     1367./1361.*[1.,1.], width=2, color="black";
  plg, 1.e-6*.5*[1.2,1.], 1367./1361.+[-dl,0.], width=2, color="black";
  plg, 1.e-6*.5*[1.2,1.], 1367./1361.+[+dl,0.], width=2, color="black";
  plt, "R+06", 1367./1361., 2.1e-6, height=8, tosys=1, justify="CH";

  // Voigt & Marotzke (2010): AOGCM (ECHAM5/MPI-OM), modern snowballs,
  // pre-industrial GHG concentrations: CO2 = 286.2 ppm, CH4 = 805.6
  // ppb, N2O = 276.7 ppb; bifurcation between 91% and 94% of the
  // present-day solar constant of 1367 W/m2:
  forc = co2_c2f_old( 286.2 ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plmk, co2e/1.e6, 0.925*1367./1361., marker=1, msize=.45, color="white";
  plmk, co2e/1.e6, 0.925*1367./1361., marker=1, msize=.4, color="black";
  plg, co2e/1.e6*[1,1], [0.91,0.94]*1367./1361., color="black";
  plg, co2e/1.e6*[0.875,1.125], 0.91*1367./1361.*[1,1], color="black";
  plg, co2e/1.e6*[0.875,1.125], 0.94*1367./1361.*[1,1], color="black";
  plt, "VM10", 0.925*1367./1361., 7.5e-4, height=8, tosys=1, justify="CH";
  plg, [3.5e-4,5.5e-4], 0.925*1367./1361.*[1,1], width=1;
  // they also have a Snowball state for a present-day solar constant
  // of 1367 W/m2 and 0.1 ppm of CO2:
  forc = co2_c2f_old( 0.1 ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plmk, co2e/1.e6, 1367./1361., marker=1, msize=.4, color="black";
  plg, co2e/1.e6*[1.,2.],     1367./1361.*[1.,1.], width=2, color="black";
  plg, co2e/1.e6*2.*[0.8,1.], 1367./1361.+[-dl,0.], width=2, color="black";
  plg, co2e/1.e6*2.*[0.8,1.], 1367./1361.+[+dl,0.], width=2, color="black";
  plt, "VM10", 1367./1361., 3.25e-7, height=8, tosys=1, justify="CH";

  // Voigt et al. (2011): AOGCM (ECHAM5/MPI-OM), Marinoan continental
  // configuration, but pre-industrial GHG concentrations: CO2 = 278
  // ppm, CH4 = 650 ppb, N2O = 270 ppb; bifurcation between 95.5% and
  // 96% of the present-day solar constant of 1367 W/m2:
  plmk, 278./1.e6, 0.9575*1367./1361., marker=1, msize=.4, color="black";
  plg, 278./1.e6*[1,1], [0.955,0.96]*1367./1361.;
  plg, 278./1.e6*[0.875,1.125], 0.955*1367./1361.*[1,1];
  plg, 278./1.e6*[0.875,1.125], 0.96*1367./1361.*[1,1];
  
  // Yang et al. (2012a): AOGCM (CCSM3), modern snowballs, solar
  // constant 1367 W/m2, 286 ppm of CO2, CH4 at 805.6 ppb, N2O at
  // 276.7 ppb, snowball bifurcation between 90% and 89.5% of the
  // present-day solar constant of 1367 W/m2:  
  forc = co2_c2f_old( 286. ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plmk, co2e/1.e6, 0.8975*1367./1361., marker=1, msize=.4, color="black";
  plg, co2e/1.e6*[1,1], [0.895,0.9]*1367./1361., color="black";
  plg, co2e/1.e6*[0.875,1.125], 0.895*1367./1361.*[1,1], color="black";
  plg, co2e/1.e6*[0.875,1.125], 0.9*1367./1361.*[1,1], color="black";
  plt, "Y+12a", 0.888*1367./1361., 1.6e-4, height=8, tosys=1, justify="CH";
  plg, [2.1e-4,2.5e-4], [0.894,0.897], width=1;


  

  // Yang et al. (2012c): AOGCM (CCSM4), modern snowballs, solar
  // constant 1367 W/m2, 286 ppm of CO2, CH4 at 805.6 ppb, N2O at
  // 276.7 ppb, snowball bifurcation between 92% and 91% of the
  // present-day solar constant of 1367 W/m2:
  // total forcing:
  forc = co2_c2f_old( 286. ) + ch4_c2f_old( 805.6 );
  // CO2 equivalent:
  co2e = co2_f2c_old( forc );
  plmk, co2e/1.e6, 0.915*1367./1361., marker=1, msize=.45, color="white";
  plmk, co2e/1.e6, 0.915*1367./1361., marker=1, msize=.4, color="black";
  plg, co2e/1.e6*[1,1], [0.91,0.92]*1367./1361., color="black";
  plg, co2e/1.e6*[0.875,1.125], 0.91*1367./1361.*[1,1], color="black";
  plg, co2e/1.e6*[0.875,1.125], 0.92*1367./1361.*[1,1], color="black";
  plt, "Y+12c", 0.915*1367./1361., 9e-5, height=8, tosys=1, justify="CH";
  plg, [1.2e-4,2.1e-4], 0.915*1367./1361.*[1,1], width=1;


}





func glac_figure2( void )
{

  // define the plot limits on the x-axis (important for labels on top
  // of the plot!):
  xl = 0.73;
  xr = 1.02;

  // read data from file:
  local tcrit, lcrit, ccrit, fice, temp;
  read_columns, "ModelOutput/critical_states_characteristics.dat",
    1,tcrit, 2,lcrit, 3,ccrit, 5,fice, 6,temp;
  
  
  // create panels for plotting:
  //  window, style="notsoflat.gs";   // only for 1st shot, replace by place_plsys!
  //  viewport notsoflat.gs:   viewport= { 0.14, 0.75, 0.45, 0.90 }
  fma;
  // upper panel:
  place_plsys, 1, 0.14, 0.75, 0.6013, 0.9013, tmark_u=0, tlabel_b=0,
    tlabel_height=18, fwidth=2, nmajor=9, nminor=25;
  // lower panel:
  place_plsys, 2, 0.14, 0.75, 0.30, 0.60, tmark_u=1, tmark_r=0, tmark_b=1, tlabel_b=1,
    tlabel_height=18, fwidth=2, nmajor=9, nminor=25;

  // UPPER PANEL: GLOBAL MEAN TEMPERATURE
  plsys, 1;
  // mark important ages on top of the plot:
  s = array( 0, [2,1,1] );
  //  pli, bytscl( s, cmin=0, cmax=1),
  //    gough_age_to_L(1.0), 0.5, gough_age_to_L(0.542), 1.0;
  ages = [4.0,3.5,3.0,2.5,2.0,1.5,1.0,0.5,0.];
  //  nages = ["3.8","2.5","0.5","0.0"];
  nages = ["4.0","3.5","3.0","2.5","2.0","1.5","1.0","0.5","0.0"];
  // convert this into relative solar luminosities:
  lages = gough_age_to_L( ages );
  pldj, lages, array( 1.4, numberof(ages) ), lages, array( 2., numberof(ages) );
  for( n=1; n<=numberof(ages); n++ )
    // viewport= { 0.14, 0.75, 0.45, 0.90 }
    plt, nages(n), 0.14+(lages(n)-xl)/(xr-xl)*(0.75-0.14), 0.905, 
      tosys=0, height=18, justify="CB";
  plg, [2.,2.], [xl,xr];
  ages = [3.8,2.5,0.542,0.];
  lages = gough_age_to_L( ages );
  // stage colors:
  // Hadean:
  ccc = char([174,2,126]);
  plf, array(ccc,2,2), [[2.99,2.99],[2.03,2.03]],
    [[xl,lages(1)],[xl,lages(1)]];
  // Archean:
  ccc = char([240,4,127]);
  plf, array(ccc,2,2), [[2.99,2.99],[2.03,2.03]],
    [[lages(1),lages(2)],[lages(1),lages(2)]];
  // Proterozoic:
  ccc = char([247,53,99]);
  plf, array(ccc,2,2), [[2.99,2.99],[2.03,2.03]],
    [[lages(2),lages(3)],[lages(2),lages(3)]];
  // Phanerozoic:
  ccc = char([154,216,221]);
  plf, array(ccc,2,2), [[2.99,2.99],[2.03,2.03]],
    [[lages(3),lages(4)],[lages(3),lages(4)]];
  pldj, lages, array( 2.99, 4 ), lages, array( 2., 4 );
  plt, "Archean", (lages(1)+lages(2))/2., 2.55, 
    tosys=1, height=12, justify="CH";
  plt, "Proterozoic", (lages(2)+lages(3))/2., 2.55, 
    tosys=1, height=12, justify="CH";
  plt, "Phanerozoic", (lages(3)+lages(4))/2., 2.55, 
    tosys=1, height=12, justify="CH";

  // shading of temperature ranges:
  w1 = where( temp<0. & temp>-5. );
  //  t1min = min(temp(w1));
  //  t1max = max(temp(w1));
  //  t1min; t1max; avg(temp(w1)); stddev(temp(w1));
  t1avg = avg(temp(w1));
  t1sig = stddev(temp(w1));
  t1min = t1avg - 3.*t1sig;
  t1max = t1avg + 3.*t1sig;
  w2 = where( temp<-12. & temp>-20. );
  //  t2min = min(temp(w2));
  //  t2max = max(temp(w2));
  t2avg = avg(temp(w2));
  t2sig = stddev(temp(w2));
  t2min = t2avg - 3.*t2sig;
  t2max = t2avg + 3.*t2sig;
  // compute solar luminosity at transition point:
  xtrans = gough_age_to_L( 1.95 );
  //  pli, array( 0.8, 2, 2 ), 0.75, t1min, 0.888, t1max, cmin=0., cmax=1.;
  pli, array( 0.8, 2, 2 ), 0.75, t1min, 1.00, t1max, cmin=0., cmax=1.;
  pli, array( 0.8, 2, 2 ), 0.90, t2min, 1.00, t2max, cmin=0., cmax=1.;
  pli, span( 1., 0.8, 301 )(,-:1:2), 0.88, t2min, 0.90, t2max, cmin=0., cmax=1.;
  palette, "gray.gp";
  
  plmk, temp, lcrit/lcrit(1), marker=4, msize=.5, color="blue";

  // PLOT LEGENDS, LIMITS, TITLES:
  plt, "(a)", 0.745, -0.25, height=18, tosys=1;
  limits, xl, xr, -18., 3.;
  xytitles, "", 
    swrite( format="Temperature (%cC)", 176 ),
    [-0.02,0.01];
  topxtitle, "Time before present (Ga)", offset=0.04;
  // switch off ticks on top of the plot:
  //  gridxy, 0x621, 0;

  // LOWER PANEL:
  plsys, 2;

  // shading of sea-ice fraction ranges:
  w1 = where( fice<0.35 & fice>0.30 );
  f1avg = avg(fice(w1));
  f1sig = stddev(fice(w1));
  f1min = f1avg - 3.*f1sig;
  f1max = f1avg + 3.*f1sig;
  w2 = where( fice<0.55 & fice>0.47 );
  f2avg = avg(fice(w2));
  f2sig = stddev(fice(w2));
  f2min = f2avg - 3.*f2sig;
  f2max = f2avg + 3.*f2sig;
  //  pli, array( 0.8, 2, 2 ), 0.75, f1min, 0.888, f1max, cmin=0., cmax=1.;
  pli, array( 0.8, 2, 2 ), 0.75, f1min, 1.00, f1max, cmin=0., cmax=1.;
  pli, array( 0.8, 2, 2 ), 0.90, f2min, 1.00, f2max, cmin=0., cmax=1.;
  pli, span( 1., 0.8, 301 )(,-:1:2), 0.88, f2min, 0.90, f2max, cmin=0., cmax=1.;
  palette, "gray.gp";
  
  plmk, fice, lcrit/lcrit(1), marker=4, msize=.5, color="blue";

  // vertical axis limits (needed for secondary axis!):
  yl = 0.275;
  yr = 0.575;
  
  // right-hand axis for critical iceline:
  lice = [30,35,40,45];
  aice = 1.-sin( lice*pi/180. );
  pldj, array( 1.015, numberof(lice) ), aice, array( xr, numberof(lice) ), aice;
  for( n=1; n<=numberof(lice); n++ )
    // viewport= { 0.14, 0.75, 0.30, 0.60 }
    plt, swrite( format="%2d", lice(n) ), 0.76, 0.3013 + (aice(n)-yl)/(yr-yl)*0.30,                tosys=0, height=18, justify="LH";
  
  // PLOT LEGENDS, LIMITS, TITLES:
  plt, "(b)", 0.745, 0.52, height=18, tosys=1;
  limits, xl, xr, yl, yr;
  xytitles, "S/S_0_", //"Solar luminosity relative to today", 
    "Sea-ice fraction", [-0.02,0.01];
  rightytitle, swrite( format="Ice-line latitude (%c)", 176 );
  // switch on ticks on top of the plot (does not work with place_plsys somehow):
  gridxy, 0x62b, 0;


  
  // FILE OUTPUT:
  filename = "Plots/glac_figure2";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

  
}



func glac_figure3( void )
{

  // PLOT WINDOW:
  window, style="notsoflat.gs";
  fma;

  // LIMITS:
  co2min = 150;
  co2max = 650;

  // READ DATA:
  local co2, fice, temp;
  read_columns, "ModelOutput/0900Ma_states_characteristics.dat",
    3,co2, 5,fice, 6,temp;
  local fice_crit;
  read_columns, "ModelOutput/critical_states_characteristics.dat",
    5,fice_crit;


  // shading of sea-ice fraction ranges:
  w1 = where( fice_crit<0.35 & fice_crit>0.30 );
  f1avg = avg(fice_crit(w1));
  f1sig = stddev(fice_crit(w1));
  f1min = f1avg - 3.*f1sig;
  f1max = f1avg + 3.*f1sig;
  w2 = where( fice_crit<0.55 & fice_crit>0.47 );
  f2avg = avg(fice_crit(w2));
  f2sig = stddev(fice_crit(w2));
  f2min = f2avg - 3.*f2sig;
  f2max = f2avg + 3.*f2sig;
  pli, array( 0.8, 2, 2 ), co2min, f1min, co2max, f1max, cmin=0., cmax=1.;
  pli, array( 0.8, 2, 2 ), co2min, f2min, co2max, f2max, cmin=0., cmax=1.;
  palette, "gray.gp";

  plg, fice(1)*[1,1.2], co2(1)*[1,1], width=2, color="blue";
  plg, fice(1)*[1.165,1.2], co2(1)*[0.985,1], width=2, color="blue";
  plg, fice(1)*[1.165,1.2], co2(1)*[1.015,1], width=2, color="blue";
  plmk, fice(1), co2(1), marker=4, msize=.5, width=2, color="blue";
  plmk, fice(2:), co2(2:), marker=4, msize=.5, color="blue";
  limits, co2min, co2max, 0.125, 0.875;
  //  plt, "900 Ma equilibrium states", (co2min+co2max)/2., 0.9, tosys=1, height=18,
  //   justify="CH";
  plt, "900 Ma equilibrium states", 620, 0.8, tosys=1, height=18,
    justify="RH";
  plt, "Hadley state", 285, 0.53, tosys=1, height=14, justify="LH";
  plt, "Ferrel state", 325, 0.36, tosys=1, height=14, justify="LH";
  xytitles, "CO_2_ concentration (ppm)", "Sea-ice fraction", [0.,0.01];
  
  // FILE OUTPUT:
  filename = "Plots/glac_figure3";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

}


func glac_figure4( void )
{

  // read file with information about Hadley states:
  local age, co2, year1, year2, type;
  read_columns, "ModelOutput/hadley_states_characteristics.dat",
    1,age, 3,co2, 4,year1, 5,year2, 6,type;

  // restrict to transient Hadley states (type=0):
  w = where( type==0 );
  age = int(age(w)); co2 = co2(w); year1 = int(year1(w)); year2 = int(year2(w));

  // note that we take the years defined in the file only as a first
  // guess and improve the identification of the transient Hadley
  // phase, see below!

  // limits used for transient Hadley phase:
  fsi_h = 0.503;
  fsi_l = 0.497;

  pbasedir = "/p/projects/ace/feulner/";
  window, style="notsoflat.gs";

  // plot sea-ice fractions used to define the transient Hadley phase:
  plg, fsi_h*[1,1], [-100,500], width=1;
  plg, fsi_l*[1,1], [-100,500], width=1;

  // loop through transient Hadley states:
  colv = [];  // array of colour values, needed for plot legend!
  for( na=1; na<=numberof(age); na++ ){

    // construct full path name using age and CO2 value:
    if( co2(na)>1. )
      pcrit = pbasedir + swrite( format="c3beta_aqua_%04dMa_%05dppm/",
                                           age(na), int(co2(na)) );
    else
      pcrit = pbasedir + swrite( format="c3beta_aqua_%04dMa_00000p%1dppm/",
                                           age(na), int(10.*co2(na)) );

    // read global average sea-ice history file:
    d = nc_openf( pcrit+"/history_isis_mean.nc", data_si, meta_si );

    // go through transient Hadley phase plus some more...:
    year = fsi = [];
    // start and end years from file only as first estimate!
    for( ny=year1(na)-200; ny<=year2(na)+400; ny++ ){
      jr, data_si, ny;
      grow, year, ny;
      grow, fsi, data_si.con_g(1,,);
    }
    fsi = fsi(1,);

    // smooth time series using SSA and identify transient Hadley
    // phase from the smoothed time series of the sea-ice fraction:
    //    fsi_s = [];
    //    ssatrend, year, fsi,  9, fsi_s;
    //    w = where( fsi_s>0.499 & fsi_s<0.501 );
    //    w = where( fsi_s>0.4985 & fsi_s<0.4995 );
    //    w = where( fsi_s>fsi_l & fsi_s<fsi_h );
   
    // set colour:
    //    _colv = (9-na+1)*20;
    _colv = na*20;
    col = _colv*[1,1,1];
    grow, colv, _colv;

    // identify beginning of transient Hadley phase:
    w = where( fsi<=fsi_l );
    year_beg = year( w(0 ) ) + 1;
    // identify end of transient Hadley phase:
    w = where( fsi>=fsi_h );
    year_end = year( w(1) );// - 1;

    // plot entire path as a thin line:
    plg, fsi, year-year_beg, width=3, color=col;
    // plot transient Hadley phase as a thicker line:
    //    w = where( year>=year_beg & year<=year_end );
    //    plg, fsi_s(w), year(w)-year(w(1)), width=4, color=col;
    //    plg, fsi(w), year(w)-year_beg, width=4, color=col;

    // compute length spent in transient Hadley state:
    write, swrite( format="%4d   %3d", age(na), year_end-year_beg+1 );

  }

  limits, -50, 450, 0.485, 0.515;
  //  plt, "Transient Hadley states", -142, 0.506, tosys=1, height=18,
  //    justify="LH";
  xytitles, "Years after entering transient Hadley state", "Sea-ice fraction", [-0.02,0.01];
  // legend:
  plg, 0.493*[1,1], [75,100], width=3, color=colv(9)*[1,1,1];
  plg, 0.491*[1,1], [75,100], width=3, color=colv(8)*[1,1,1];
  plg, 0.489*[1,1], [75,100], width=3, color=colv(7)*[1,1,1];
  plt, "3600 Ma", 110, 0.493, tosys=1, height=14, justify="LH";
  plt, "3300 Ma", 110, 0.491, tosys=1, height=14, justify="LH";
  plt, "3000 Ma", 110, 0.489, tosys=1, height=14, justify="LH";
  
  plg, 0.493*[1,1], [200,225], width=3, color=colv(6)*[1,1,1];
  plg, 0.491*[1,1], [200,225], width=3, color=colv(5)*[1,1,1];
  plg, 0.489*[1,1], [200,225], width=3, color=colv(4)*[1,1,1];
  plt, "2700 Ma", 235, 0.493, tosys=1, height=14, justify="LH";
  plt, "2400 Ma", 235, 0.491, tosys=1, height=14, justify="LH";
  plt, "2100 Ma", 235, 0.489, tosys=1, height=14, justify="LH";

  plg, 0.493*[1,1], [325,350], width=3, color=colv(3)*[1,1,1];
  plg, 0.491*[1,1], [325,350], width=3, color=colv(2)*[1,1,1];
  plg, 0.489*[1,1], [325,350], width=3, color=colv(1)*[1,1,1];
  plt, "1650 Ma", 360, 0.493, tosys=1, height=14, justify="LH";
  plt, "1500 Ma", 360, 0.491, tosys=1, height=14, justify="LH";
  plt, "1350 Ma", 360, 0.489, tosys=1, height=14, justify="LH";

    // FILE OUTPUT:
  filename = "Plots/glac_figure4";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

}




func glac_figure5ac( void )
{

  // PANELS FOR PLOTTING:
  fma;

  /*
  // upper panel:
  place_plsys, 1, 0.15, 0.75, 0.6013, 0.9013, ftype=0, tlabel_l=0, tlabel_b=0;
  // lower panel:
  place_plsys, 2, 0.15, 0.75, 0.28, 0.58, ftype=0, tlabel_l=0, tlabel_b=0;
  */
  // upper panel:
  place_plsys, 1, 0.15, 0.45, 0.7513, 0.9013, ftype=0, tlabel_l=0, tlabel_b=0;
  // lower panel:
  place_plsys, 2, 0.15, 0.45, 0.52, 0.67, ftype=0, tlabel_l=0, tlabel_b=0;

  
  // READ DATA:
  pbasedir = "ModelOutput/";
  // atmosphere history files:
  d1 = nc_openf( pbasedir+"3000Ma_32600ppm_history_p2.ts_ann.timmean.nc",
                 data_at1, meta_at1 );
  d2 = nc_openf( pbasedir+"0900Ma_00240ppm_history_p2.ts_ann.timmean.nc",
                 data_at2, meta_at2 );
  // temperature data on atmosphere grid:
  temp1 = data_at1.ts_ann(,,1);
  temp2 = data_at2.ts_ann(,,1);

  // compute global minimum and maximum for scaling:
  mint = min(min(temp1),min(temp2));
  maxt = max(max(temp1),max(temp2));

  // construct CLIMBER-3alpha atmosphere grid (corners):
  lat_a = span( -90., +90., 25 );
  lon_a = span( -180., 180., 17 );

  
  // compute Mollweide transformation matrix for the atmosphere grid:
  m_data_a = compute_mollweide_transformation( lon_a, lat_a );
  // manually adjust to avoid plotting artifacts near the poles:
  m_data_a(,1,2) = -1.01;
  m_data_a(,0,2) =  1.01;

 
  // UPPER PANEL: 
  plsys, 1;

  // surface air temperature:
  plf, temp1(1:8,13:24), m_data_a(1:9,13:25,2), m_data_a(1:9,13:25,1), cmin=mint, cmax=maxt;
  plf, temp1(1:8,13:24), m_data_a(1:9,13:25,2)-0.0001, m_data_a(1:9,13:25,1), cmin=mint, cmax=maxt;
  plf, temp1(1:8,13:24), m_data_a(1:9,13:25,2)+0.0001, m_data_a(1:9,13:25,1), cmin=mint, cmax=maxt;
  plf, temp1(1:8,13:24), m_data_a(1:9,13:25,2), m_data_a(1:9,13:25,1)-0.0001, cmin=mint, cmax=maxt;
  plf, temp1(1:8,13:24), m_data_a(1:9,13:25,2), m_data_a(1:9,13:25,1)+0.0001, cmin=mint, cmax=maxt;
  palette, "bluered.gp";
  
  // grid, edges, limits, legends:
  plot_mollweide_grid, color=100*[1,1,1], width=1;
  plot_mollweide_edge, color="black", width=3;
  limits, -2.1, 0., 0., 1.05;
  plt, "(a)", -1.95, .9, height=18, tosys=1, justify="LH";
  //  plt, "3000 Ma", 2.15, 0., height=18, tosys=1, justify="RH";
  plt, "3000 Ma", 0.52, 0.8263, height=18, tosys=0, justify="CH";


  // LOWER PANEL: 
  plsys, 2;

  // surface air temperature:
  plf, temp2(1:8,1:12), m_data_a(1:9,1:13,2),        m_data_a(1:9,1:13,1), cmin=mint, cmax=maxt;
  plf, temp2(1:8,1:12), m_data_a(1:9,1:13,2)-0.0001, m_data_a(1:9,1:13,1), cmin=mint, cmax=maxt;
  plf, temp2(1:8,1:12), m_data_a(1:9,1:13,2)+0.0001, m_data_a(1:9,1:13,1), cmin=mint, cmax=maxt;
  plf, temp2(1:8,1:12), m_data_a(1:9,1:13,2),        m_data_a(1:9,1:13,1)-0.0001, cmin=mint, cmax=maxt;
  plf, temp2(1:8,1:12), m_data_a(1:9,1:13,2),        m_data_a(1:9,1:13,1)+0.0001, cmin=mint, cmax=maxt;

  // grid, edges, limits, legends:
  plot_mollweide_grid, color=100*[1,1,1], width=1;
  plot_mollweide_edge, color="black", width=3;

  limits, -2.1, 0, -1.05, 0; 
  plt, "(c)", -1.95, -.85, height=18, tosys=1, justify="LH";
  //  plt, "900 Ma", 1.95, .9, height=18, tosys=1, justify="RH";
  plt, "900 Ma", 0.52, .595, height=18, tosys=0, justify="CH";

  colorbar_, mint, maxt, sys=3, viewport=[0.21,0.41,0.71,0.72], nmajor=9, nminor=5,
    tlabel_b=1, horizontal=1, tlabel_height=10, ftype=1;
  plt, swrite( format="Surface air temperature (%cC)", 176 ),
    0.31, 0.7325, tosys=0, justify="CH", height=10;
  
  
  // FILE OUTPUT:
  filename = "Plots/glac_figure5ac";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

}




func glac_figure5bd( void )
{

  fma;
  
  // upper panel:
  place_plsys, 1, 0.15, 0.45, 0.7513, 0.9013, ftype=0, tlabel_l=0, tlabel_b=0;
  // lower panel:
  place_plsys, 2, 0.15, 0.45, 0.52, 0.67, ftype=0, tlabel_l=0, tlabel_b=0;

  // READ DATA:
  pbasedir = "ModelOutput/";
  // sea-ice history files:
  d1 = nc_openf( pbasedir+"3000Ma_32600ppm_history_isis.con_g.timmean.nc",
                 data_si1, meta_si1 );
  d2 = nc_openf( pbasedir+"0900Ma_00240ppm_history_isis.con_g.timmean.nc",
                 data_si2, meta_si2 );
  //  ensemble means of atmosphere snapshots:
  d3 = nc_openf( "ModelOutput/3000Ma_32600ppm_snapshots_potsdam2.timmean.nc", data_a1, meta_a1 );
  d4 = nc_openf( "ModelOutput/0900Ma_00240ppm_snapshots_potsdam2.timmean.nc", data_a2, meta_a2 );
  sice1 = data_si1.con_g(,,1);
  sice2 = data_si2.con_g(,,1);
  // extract surface wind fields (no units in netCDF files, should be m/s):
  u1 = data_a1.u3d(,,1);
  v1 = data_a1.v3d(,,1);
  u2 = data_a2.u3d(,,1);
  v2 = data_a2.v3d(,,1);
  // compute wind speeds (no units in netCDF files, should be m/s):
  wind1 = sqrt( u1^2 + v1^2 );
  wind2 = sqrt( u2^2 + v2^2 );
  min(wind1); avg(wind1); max(wind1);
  min(wind2); avg(wind2); max(wind2);
  
  // construct CLIMBER-3alpha ocean grid (corners):
  lat_o = span(  -90., +90., 49 );
  lon_o = span( -180., 180., 97 );
  // atmosphere grid centers:
  lat_a = span( -86.25, +86.25, 24 );
  lon_a = span( -168.75, 168.75, 16 );
  // 2D arrays for the atmosphere grid:
  lat_a2 = lat_a(-:1:numberof(lon_a),);
  lon_a2 = lon_a(,-:1:numberof(lat_a));

  // compute Mollweide transformation matrix for the ocean grid (corners):
  m_data_o = compute_mollweide_transformation( lon_o, lat_o );
  // manually adjust to avoid plotting artifacts near the poles:
  m_data_o(,1,2) = -1.01;
  m_data_o(,0,2) =  1.01;
  // compute Mollweide transformation matrix for the atmosphere grid (centers):
  m_data_a = compute_mollweide_transformation( lon_a, lat_a );


  // UPPER PANEL: 
  plsys, 1;

  // sea-ice fraction:
  plf, sice1(49:96,25:48), m_data_o(49:97,25:49,2),        m_data_o(49:97,25:49,1), cmin=0, cmax=1;
  plf, sice1(49:96,25:48), m_data_o(49:97,25:49,2)-0.0001, m_data_o(49:97,25:49,1);
  plf, sice1(49:96,25:48), m_data_o(49:97,25:49,2)+0.0001, m_data_o(49:97,25:49,1);
  plf, sice1(49:96,25:48), m_data_o(49:97,25:49,2),        m_data_o(49:97,25:49,1)-0.0001;
  plf, sice1(49:96,25:48), m_data_o(49:97,25:49,2),        m_data_o(49:97,25:49,1)+0.0001;
  r = [0]; grow, r, span(0, 220, 238);
  g = [0]; grow, g, span(0, 255, 238);
  b = [125]; grow, b, span( 125, 255, 238 );
  palette, r,g,b;

  // surface wind field:
  // atmosphere grid centre positions:
  for( iii=1; iii<=16; iii++ )
    plmk, m_data_a(iii,,2), m_data_a(iii,,1), marker=4, msize=.1, color=[100,100,100];

  // first rough test wind field:
  scale = 2.; 
  // add scaled wind-field components to coordinates:
  e_data_a = compute_mollweide_transformation_array( lon_a2+scale*u1,
                                                     lat_a2+scale*v1 );
  pldj, m_data_a(9:16,13:24,1), m_data_a(9:16,13:24,2), e_data_a(9:16,13:24,1), e_data_a(9:16,13:24,2), color=[150,150,150];

  // arrow indicating wind scaling:
  plmk, -1.03, 1.6, marker=4, msize=.1;
  plg, -1.03*[1,1], 1.6+[0.,0.222222];
  plt, "10 m s^-1^", 1.711111, -0.9, tosys=1, height=10, justify="CH";

  
  // grid, edges, limits, legends:
  plot_mollweide_grid, color=100*[1,1,1], width=1;
  plot_mollweide_edge, color="black", width=3;
  limits, 0, 2.1, -0, 1.05;
  plt, "(b)", 1.95, .9, height=18, tosys=1, justify="RH";
  //  plt, "3000 Ma", 1.95, .9, height=18, tosys=1, justify="RH";


  // LOWER PANEL: 
  plsys, 2;

  // sea-ice fraction:
  plf, sice2(49:96,1:48), m_data_o(49:97,1:49,2),        m_data_o(49:97,1:49,1), cmin=0, cmax=1;
  plf, sice2(49:96,1:48), m_data_o(49:97,1:49,2)-0.0001, m_data_o(49:97,1:49,1);
  plf, sice2(49:96,1:48), m_data_o(49:97,1:49,2)+0.0001, m_data_o(49:97,1:49,1);
  plf, sice2(49:96,1:48), m_data_o(49:97,1:49,2),        m_data_o(49:97,1:49,1)-0.0001;
  plf, sice2(49:96,1:48), m_data_o(49:97,1:49,2),        m_data_o(49:97,1:49,1)+0.0001;

  // surface wind field:
  // test atm. grid centre positions:
  for( iii=1; iii<=16; iii++ )
    plmk, m_data_a(iii,,2), m_data_a(iii,,1), marker=4, msize=.1, color=[100,100,100];

  // first rough test wind field:
  scale = 2.; 
  // add scaled wind-field components to coordinates:
  e_data_a = compute_mollweide_transformation_array( lon_a2+scale*u2,
                                                     lat_a2+scale*v2 );
  pldj, m_data_a(9:16,1:12,1), m_data_a(9:16,1:12,2), e_data_a(9:16,1:12,1), e_data_a(9:16,1:12,2), color=[150,150,150];

  // arrow indicating wind scaling:
  plmk, -0.98, 1.4, marker=4, msize=.1;
  plg, -0.98*[1,1], 1.4+[0.,0.222222];
  plt, "10 m s^-1^", 1.511111, -0.9, tosys=1, height=10, justify="CH";

  // grid, edges, limits, legends:
  plot_mollweide_grid, color=100*[1,1,1], width=1;
  plot_mollweide_edge, color="black", width=3;
  limits, 0, 2.1, -1.05, 0; 
  plt, "(d)", 1.95, -.85, height=18, tosys=1, justify="RH";
  //  plt, "900 Ma", 1.95, .9, height=18, tosys=1, justify="RH";

  colorbar_, 0, 1, sys=3, viewport=[0.195,0.395,0.71,0.72], nmajor=3, nminor=0,
    tlabel_b=1, horizontal=1, tlabel_height=10, ftype=1;
  plt, "Sea-ice fraction", 0.295, 0.7325, tosys=0, justify="CH", height=10;

  

  
  // FILE OUTPUT:
  filename = "Plots/glac_figure5bd";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

}









func glac_figure6( void )
{

  // READ DATA:
  // surface air temperature:
  pbasedir = "ModelOutput/";
  d1 = nc_openf( pbasedir+"3000Ma_32600ppm_history_p2.ts_ann.timmean.nc",
                 data_at1, meta_at1 );
  d2 = nc_openf( pbasedir+"0900Ma_00240ppm_history_p2.ts_ann.timmean.nc",
                 data_at2, meta_at2 );
  // sea-ice fractions:
  d3 = nc_openf( pbasedir+"3000Ma_32600ppm_history_isis.con_g.timmean.nc",
                 data_si1, meta_si1 );
  d4 = nc_openf( pbasedir+"0900Ma_00240ppm_history_isis.con_g.timmean.nc",
                 data_si2, meta_si2 );
  // meridional surface wind:
  d5 = nc_openf( pbasedir+"3000Ma_32600ppm_snapshots_potsdam2.timmean.nc",
                 data_wd1, meta_wd1 )
  d6 = nc_openf( pbasedir+"0900Ma_00240ppm_snapshots_potsdam2.timmean.nc",
                 data_wd2, meta_wd2 )

  // compute zonal means:
  temp1z = data_at1.ts_ann(avg,);
  temp2z = data_at2.ts_ann(avg,);
  sice1z = data_si1.con_g(avg,,1);
  sice2z = data_si2.con_g(avg,,1);
  wind1z = data_wd1.v3d(avg,,1);
  wind2z = data_wd2.v3d(avg,,1);


  lat_o = span( -88.125, +88.125, 48 );
  lat_a = span( -86.25, +86.25, 24 );


  
  // PANELS FOR PLOTTING:
  window, style="threepanels_new.gs";
  fma;

  pltitle_height=14;


  // equal-area scaling: sin(phi*pi/180.)*90.
  
  plsys, 1;
  tmin = -57.5; tmax = 17.5;
  plg, [tmin,tmax], [-42,-42], width=2, color="blue", type="dash";
  plg, [tmin,tmax], [+42,+42], width=2, color="blue", type="dash";
  plg, [tmin,tmax], [-30,-30], width=2, color="blue";
  plg, [tmin,tmax], [+30,+30], width=2, color="blue";
  plg, temp1z, lat_a, width=3, color="red", type="dash";
  plg, temp2z, lat_a, width=3, color="red";
  limits, -90, 90, tmin, tmax;
  xytitles, "", swrite( format="Temperature (%cC)", 176 );
  plt, "(a)", -81.5, 5, height=14, tosys=1;
  plegend, ["dash","solid"], ["3000 Ma","900 Ma"], width=3,
    color="red", origin=[0.76,0.96], length=0.024;

  plsys, 2;
  imin = -0.1; imax = 1.1;
  plg, [imin,imax], [-42,-42], width=2, color="blue", type="dash";
  plg, [imin,imax], [+42,+42], width=2, color="blue", type="dash";
  plg, [imin,imax], [-30,-30], width=2, color="blue";
  plg, [imin,imax], [+30,+30], width=2, color="blue";
  plg, sice1z, lat_o, width=3, color="red", type="dash";
  plg, sice2z, lat_o, width=3, color="red";
  limits, -90, 90, imin, imax;
  xytitles, "", "Sea-ice fraction";
  plt, "(b)", -81.5, 0.875, height=14, tosys=1;

  plsys, 3;
  wmin = -1.675; wmax = +1.675;
  plg, [wmin,wmax], [-42,-42], width=2, color="blue", type="dash";
  plg, [wmin,wmax], [+42,+42], width=2, color="blue", type="dash";
  plg, [wmin,wmax], [-30,-30], width=2, color="blue";
  plg, [wmin,wmax], [+30,+30], width=2, color="blue";
  plg, [0,0], [-90,90];
  plg, wind1z, lat_a, width=3, color="red", type="dash";
  plg, wind2z, lat_a, width=3, color="red";
  limits, -90, 90, wmin, wmax;
  xytitles, swrite( format="Latitude (%c)", 176 ), "Meridional wind (ms^-1^)", [0,0.02];
  gridxy, base60=1;
  plt, "(c)", -81.5, 1.15, height=14, tosys=1;
  
  // FILE OUTPUT:
  filename = "Plots/glac_figure6";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

}


func glac_figure7( void )
{

  // create panels for plotting (copied from glac_figure2):
  fma;
  // upper panel:
  place_plsys, 1, 0.14, 0.75, 0.6013, 0.9013, tmark_l=0, tlabel_l=0, tlabel_r=0,
    tmark_r=0, tmark_b=0, tlabel_b=0, tmark_u=0, tlabel_u=0, tlabel_height=14,
    ftype=1, fwidth=2, nmajor=9, nminor=25;
  // lower panel:
  place_plsys, 2, 0.14, 0.75, 0.225, 0.525, tmark_u=1, tmark_r=0, tmark_b=0, tlabel_b=0,
    tlabel_l=0, tmark_l=0, tlabel_height=14, ftype=1, fwidth=2, nmajor=9, nminor=25;

  // UPPER PANEL: FERREL STATE
  plsys, 1;
  // ocean:
  ccc = char([0,0,175]);
  plf, array(ccc,2,2), [[0,0],[-3,-3]], [[90,0],[90,0]];
  // Hadley and Ferrel cell boundaries:
  plg, [0,8], sin(30.*pi/180.)*90*[1,1], type="dot", width=3;
  plg, [0,6], sin(60.*pi/180.)*90*[1,1], type="dot", width=3;
  // cell names:
  plt, "Hadley cell", 0.25*90, 4.85, tosys=1, height=14,
    justify="CH", font="helveticaI";
  plt, "Ferrel cell", 0.685*90, 3.85, tosys=1, height=14,
    justify="CH", font="helveticaI";
  // wind arrows Hadley cell:
  phi = span( 0., 2.*pi, 1000 );
  plg, 4.75+3.75*sin(phi), (0.25+0.20*cos(phi))*90., width=2;
  plg, [1.15,1],  [0.26,0.24]*90., width=2;
  plg, [0.85,1],  [0.26,0.24]*90., width=2;
  plg, [8.65,8.5],  [0.24,0.26]*90., width=2;
  plg, [8.35,8.5],  [0.24,0.26]*90., width=2;
  plg, [4.5,5],   [0.056,0.05]*90., width=2;
  plg, [4.5,5],   [0.044,0.05]*90., width=2;
  plg, [5,4.5],   [0.456,0.45]*90., width=2;
  plg, [5,4.5],   [0.444,0.45]*90., width=2;
  
  // wind arrows Ferrel cell:
  plg, 3.75+2.75*sin(phi), (0.685+0.135*cos(phi))*90., width=2;
  plg, [1.15,1],  [0.675,0.695]*90., width=2;
  plg, [0.85,1],  [0.675,0.695]*90., width=2;
  plg, [6.65,6.5],  [0.695,0.675]*90., width=2;
  plg, [6.35,6.5],  [0.695,0.675]*90., width=2;
  plg, [4.,3.5],  [0.556,0.55]*90., width=2;
  plg, [4.,3.5],  [0.544,0.55]*90., width=2;
  plg, [3.5,4.],  [0.826,0.82]*90., width=2;
  plg, [3.5,4.],  [0.814,0.82]*90., width=2;


  // sea ice:
  // the thickness distribution in the Ferrel states falls from about
  // 80 m at the pole to about 2 m at 60 degrees and then decreases
  // linearly reaching zero at about 35 degrees; here, we use 5 m
  // rather than 2 m because it is hard to see otherwise; let's first
  // construct the thickness distribution:
  lice = span( 90., 30., 8000 );
  dice = lice*0.;
  w = where( lice<=60. & lice>=35. );
  dice(w) = 5.*( (lice(w)-35.)/25. ); // [m] linearly decreasing thickness within the Ferrel cell
  w = where( lice>60. );
  dice(w) = 5.+75.*sqrt( (lice(w)-60.)/30. ); // [m]
  // now we have to take into account that 7/8th of ice is below sea
  // level:
  ccc = char([0,225,225]);
  lice = sin(lice*pi/180.)*90.;
  plf, array(ccc,numberof(lice),2),
    [dice/8.,-7.*dice/8.]/25., [lice,lice];
  plg, [0.25,0.75], [1,1]*sin(42./180.*pi)*90., width=4, color=ccc;
  // limits, axis ticks & labels etc.:
  limits, 90, 0, -3, 10;
  plt, "(a)  Ferrel state", 86.5, 8.5, height=18, tosys=1, justify="LH";
  pltitle_height = 14;
  xytitles, swrite( format="Latitude (%c)", 176 ), "", [0.,0.02];
  ticks = [0,15,30,45,60,75,90];
  sticks = sin(ticks*pi/180.)*90.;
  pldj, sticks, array( -3, numberof(ticks) ), sticks, array( -2.6, numberof(ticks) );
  for( n=1; n<=numberof(ticks); n+=2 )
    // viewport= { 0.14, 0.75, 0.6013, 0.9013 }
    plt, swrite( format="%2d", ticks(n) ), 0.14 + (90.-sticks(n))/90.*0.61, 0.585,                tosys=0, height=14, justify="CH";


  

  // LOWER PANEL: HADLEY STATE
  plsys, 2;
  // ocean:
  ccc = char([0,0,175]);
  plf, array(ccc,2,2), [[0,0],[-3,-3]], [[90,0],[90,0]];
  // Hadley and Ferrel cell boundaries:
  plg, [0,8], sin(30.*pi/180.)*90*[1,1], type="dot", width=3;
  plg, [0,6], sin(60.*pi/180.)*90*[1,1], type="dot", width=3;
  // cell names:
  plt, "Hadley cell", 0.25*90, 4.85, tosys=1, height=14,
    justify="CH", font="helveticaI";
  plt, "Ferrel cell", 0.685*90, 3.85, tosys=1, height=14,
    justify="CH", font="helveticaI";
  // wind arrows Hadley cell:
  plg, 4.75+3.75*sin(phi), (0.25+0.20*cos(phi))*90., width=2;
  plg, [1.15,1],  [0.26,0.24]*90., width=2;
  plg, [0.85,1],  [0.26,0.24]*90., width=2;
  plg, [8.65,8.5],  [0.24,0.26]*90., width=2;
  plg, [8.35,8.5],  [0.24,0.26]*90., width=2;
  plg, [4.5,5],   [0.056,0.05]*90., width=2;
  plg, [4.5,5],   [0.044,0.05]*90., width=2;
  plg, [5,4.5],   [0.456,0.45]*90., width=2;
  plg, [5,4.5],   [0.444,0.45]*90., width=2;
  // wind arrows Ferrel cell:
  plg, 3.75+2.75*sin(phi), (0.685+0.135*cos(phi))*90., width=2;
  plg, [1.15,1],  [0.675,0.695]*90., width=2;
  plg, [0.85,1],  [0.675,0.695]*90., width=2;
  plg, [6.65,6.5],  [0.695,0.675]*90., width=2;
  plg, [6.35,6.5],  [0.695,0.675]*90., width=2;
  plg, [4.,3.5],  [0.556,0.55]*90., width=2;
  plg, [4.,3.5],  [0.544,0.55]*90., width=2;
  plg, [3.5,4.],  [0.826,0.82]*90., width=2;
  plg, [3.5,4.],  [0.814,0.82]*90., width=2;
  // sea ice:
  // the thickness distribution in the Hadley states falls from about
  // 30 m at the pole to about 2 m at 60 degrees and remains roughly
  // constant down to 30 degrees; here, we use 5 m rather than 2 m
  // because it is hard to see otherwise; let's first construct the
  // thickness distribution:
  lice = span( 90., 30., 8000 );
  dice = lice*0.;
  w = where( lice<=60. );
  dice(w) = 5.; // [m] constant ice thickness within the Ferrel cell
  w = where( lice>60. );
  dice(w) = 5.+25.*sqrt( (lice(w)-60.)/30. ); // [m]
  // now we have to take into account that 7/8th of ice is below sea
  // level:
  ccc = char([0,225,225]);
  lice = sin(lice*pi/180.)*90.;
  plf, array(ccc,numberof(lice),2),
    [dice/8.,-7.*dice/8.]/25., [lice,lice];
  plg, [0.25,0.75], [0.5,0.5]*90., width=4, color=ccc;
  // limits, axis ticks & labels etc.:
  limits, 90, 0, -3, 10;
  plt, "(b)  Hadley state", 86.5, 8.5, height=18, tosys=1, justify="LH";
  xytitles, swrite( format="Latitude (%c)", 176 ), "", [0.,0.015];
  ticks = [0,15,30,45,60,75,90];
  sticks = sin(ticks*pi/180.)*90.;
  pldj, sticks, array( -3, numberof(ticks) ), sticks, array( -2.6, numberof(ticks) );
  for( n=1; n<=numberof(ticks); n+=2 )
    // viewport= { 0.14, 0.75, 0.225, 0.525 }
    plt, swrite( format="%2d", ticks(n) ), 0.14 + (90.-sticks(n))/90.*0.61, 0.205,                tosys=0, height=14, justify="CH";

  // FILE OUTPUT:
  filename = "Plots/glac_figure7";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

}



func glac_figure8( void )
{

  // SOME DEFINITIONS:
  
  // construct CLIMBER-3alpha latitude grid:
  lat_a = span( -86.25, +86.25, 24 );
  
  // Stefan-Boltzmann constant:
  //  sigma = 5.670374419e-8; // [W/m2/K4] precise value
  sigma = 5.67e-8;            // [W/m2/K4] value as used in model code

  // REFERENCE HADLEY STATE AT 900 Ma:
  
  // load ensemble mean of atmosphere snapshots:
  d = nc_openf( "ModelOutput/0900Ma_00240ppm_snapshots_potsdam2.timmean.nc",
                data_a2, meta_a2 );

  // extract diagnostics required for 1D EBM:
  // absorbed solar radiation [W/m2]:
  q_abs_0 = data_a2.fs2(avg,); 
  // planetary albedo [1]:
  ap_0 = data_a2.ap(avg,);
  // divergence of meridional heat transport [W/m2]:
  h_0 = data_a2.rbtp(avg,);
  // effective emissivity [1]:
  epsilon_0 = data_a2.fl3(avg,)/data_a2.fl2(avg,);
  // surface temperature [deg C]:
  //  temp_0 = data_a2.tsur(avg,);
  
  // compute EBM temperature:
  temp_ebm_0 = ( (q_abs_0-h_0)/(sigma*epsilon_0) )^0.25;
  // convert to deg C:
  //  temp_ebm_0 -= 273.15;


  // OTHER HADLEY STATES:
  ages = [0,150,300,500,700,1050,1200,1800];
  co2  = [0.1,3,11,45,100,380,570,2620];

 
  // PANELS FOR PLOTTING:
  window, style="fourpanels_new.gs";
  fma;

  pltitle_height=14;
  
  for( na=1; na<=numberof(ages); na++ ){

    // load ensemble mean of atmosphere snapshots:
    if( co2(na)<1. )
      d = nc_openf( swrite( format="ModelOutput/%04dMa_00000p%1dppm_snapshots_potsdam2.timmean.nc", int(ages(na)), int(10.*co2(na)) ), data_a2, meta_a2 );
    else
      d = nc_openf( swrite( format="ModelOutput/%04dMa_%05dppm_snapshots_potsdam2.timmean.nc", int(ages(na)), int(co2(na)) ), data_a2, meta_a2 );
    
    // extract diagnostics required for 1D EBM:
    // absorbed solar radiation [W/m2]:
    q_abs = data_a2.fs2(avg,); 
    // planetary albedo [1]:
    ap = data_a2.ap(avg,);
    // divergence of meridional heat transport [W/m2]:
    h = data_a2.rbtp(avg,);
    // effective emissivity [1]:
    epsilon = data_a2.fl3(avg,)/data_a2.fl2(avg,);
    // surface temperature [deg C]:
    //    temp = data_a2.tsur(avg,);

    // compute EBM temperature:
    temp_ebm = ( (q_abs-h)/(sigma*epsilon) )^0.25;
    // compute contributions to temperature difference...
    // ... due to absorbed solar radiation:
    dt_a =  ( (q_abs-h_0)/(sigma*epsilon_0) )^0.25 - temp_ebm_0;
    // ... due to meridional heat transport:
    dt_h =  ( (q_abs_0-h)/(sigma*epsilon_0) )^0.25 - temp_ebm_0;
    // ... due to emissivity:
    dt_e =  ( (q_abs_0-h_0)/(sigma*epsilon) )^0.25 - temp_ebm_0;

    col = (na-1)*30*[1,1,1];
    
    plsys, 1;
    plg, temp_ebm-temp_ebm_0, lat_a, width=3, color=col;

    plsys, 2;
    plg, dt_a, lat_a, width=3, color=col, type="dash";

    plsys, 3;
    plg, dt_e, lat_a, width=3, color=col, type="dashdot";

    plsys, 4;
    plg, dt_h, lat_a, width=3, color=col, type="dot";
    
  }


  // all other plotting stuff...:
  plsys, 1;
  plt, "(a)", -82.5, 1.7, tosys=1, height=14;
  plg, 1.8*[1,1], [-70,-60.], width=3, color=7*30*[1,1,1];
  plt, "1800 Ma vs. 900 Ma", -57.5, 1.7, tosys=1, height=12;
  plg, 1.8*[1,1], [0,15], width=1;
  plg, [1.9,1.8], [10,15], width=1;
  plg, [1.7,1.8], [10,15], width=1;
  plg, 1.8*[1,1], [20,30], width=3, color="black";
  plt, "0 Ma vs. 900 Ma", 32.5, 1.7, tosys=1, height=12;
  plg, [0,0], [-90,90];
  limits, -90, 90, -2.5, 2.5;
  gridxy, base60=1;
  xytitles, "", 
    swrite( format="!DT_sfc, total_ (%cC)", 176 ),
    [0.01,0.02];

  plsys, 2;
  plt, "(b)", -82.5, 6, tosys=1, height=14;
  plg, [0,0], [-90,90];
  limits, -90, 90, -9., 9.;
  gridxy, base60=1;
  xytitles, "", 
    swrite( format="!DT_sfc, solar_ (%cC)", 176 ),
    [0.01,0.02];

  plsys, 3;
  plt, "(c)", -82.5, 6, tosys=1, height=14;
  plg, [0,0], [-90,90];
  limits, -90, 90, -9., 9.;
  gridxy, base60=1;
  xytitles, "", 
    swrite( format="!DT_sfc, greenhouse_ (%cC)", 176 ),
    [0.01,0.02];

  plsys, 4;
  plt, "(d)", -82.5, 6, tosys=1, height=14;
  plg, [0,0], [-90,90];
  limits, -90, 90, -9., 9.;
  gridxy, base60=1;

  xytitles, swrite( format="Latitude (%c)", 176 ), 
    swrite( format="!DT_sfc, transport_ (%cC)", 176 ),
    [0.01,0.02];


  // FILE OUTPUT:
  filename = "Plots/glac_figure8";
  pdf, filename;
  eps, filename;
  png, filename, dpi=600;

  
}


